using System;

internal struct Point : IComparable
{
    private Int32 m_x, m_y;

    // Konstruktor fr die einfache Initialisierung der Felder
    public Point(Int32 x, Int32 y)
    {
        m_x = x;
        m_y = y;
    }

    // berschreibung der Methode ToString, die von System.ValueType geerbt wurde
    public override String ToString()
    {
        // Den Point als String zurckgeben.
        return String.Format("({0}, {1})", m_x, m_y);
    }

    // Typsichere Implementierung der CompareTo-Methode
    public Int32 CompareTo(Point other)
    {
        // Mithilfe des Satzes des Pythagoras berechnen, welcher
        // Punkt weiter vom Ursprung (0, 0) entfernt ist.
        return Math.Sign(Math.Sqrt(m_x * m_x + m_y * m_y)
           - Math.Sqrt(other.m_x * other.m_x + other.m_y * other.m_y));
    }

    // Implementierung der Methode CompareTo von IComparable
    public Int32 CompareTo(Object o)
    {
        if (GetType() != o.GetType())
        {
            throw new ArgumentException("o is not a Point");
        }
        // Typsichere CompareTo-Methode aufrufen
        return CompareTo((Point)o);
    }
}

public static class Program
{
    public static void Main()
    {
        // Zwei Point-Instanzen auf dem Stack anlegen.
        Point p1 = new Point(10, 10);
        Point p2 = new Point(20, 20);

        // p1 wird beim Aufruf von ToString (eine virtuelle Methode) NICHT geschachtelt.
        Console.WriteLine(p1.ToString()); // "(10, 10)"

        // p1 WIRD beim Aufruf von GetType (eine nicht virtuelle Methode) geschachtelt.
        Console.WriteLine(p1.GetType()); // "Point"

        // p1 wird beim Aufruf von CompareTo NICHT geschachtelt.
        // p2 wird NICHT geschachtelt, weil CompareTo(Point) aufgerufen wird.
        Console.WriteLine(p1.CompareTo(p2)); // "-1"

        // p1 WIRD geschachtelt, und der Verweis wird in c gespeichert.
        IComparable c = p1;
        Console.WriteLine(c.GetType()); // "Point"

        // p1 wird beim Aufruf von CompareTo NICHT geschachtelt.
        // Da CompareTo keine Point-Variable bergeben bekommt,
        // wird CompareTo(Object) aufgerufen, das einen Verweis auf ein
        // geschachteltes Point bentigt.
        // c wird NICHT geschachtelt, weil es bereits auf ein
        // geschachteltes Point verweist.
        Console.WriteLine(p1.CompareTo(c)); // "0"

        // c wird NICHT geschachtelt, weil es bereits auf ein
        // geschachteltes Point verweist.
        // p2 wird geschachtelt, weil CompareTo(Object) aufgerufen wird.
        Console.WriteLine(c.CompareTo(p2)); // "-1"

        // c wird entschachtelt und die Felder werden in p2 kopiert.
        p2 = (Point)c;

        // Beweis, dass die Felder in p2 kopiert wurden.
        Console.WriteLine(p2.ToString()); // "(10, 10)"
    }
}
