/******************************************************************************
Module:  BitArray.cs
Notices: Copyright (c) 2005 Jeffrey Richter
******************************************************************************/


using System;


///////////////////////////////////////////////////////////////////////////////


public sealed class BitArray
{
   // Privates Array aus Bytes. Hier werden die Bits gespeichert.
   private Byte[] m_byteArray;
   private Int32 m_numBits;

   // Der Konstruktor legt das Bytearray an und initialisiert alle Bits mit 0
   public BitArray(Int32 numBits)
   {
      // Zuerst die Parameter berprfen.
      if (numBits <= 0)
         throw new ArgumentOutOfRangeException("numBits",
            numBits.ToString(),
            "numBits must be > 0");

      // Anzahl der Bits speichern.
      m_numBits = numBits;

      // Die Bytes zum Speichern des Bitarrays reservieren.
      m_byteArray = new Byte[(numBits + 7) / 8];
   }

   // Dies ist der Indexer (Eigenschaft mit Parametern).
   public Boolean this[Int32 bitPos]
   {

      // Dies ist die get-Accessormethode des Indexers.
      get
      {
         // Zuerst die Parameter berprfen
         if ((bitPos < 0) || (bitPos >= m_numBits))
            throw new ArgumentOutOfRangeException("bitPos");

         // Zustand des indizierten Bits zurckgeben.
         return (m_byteArray[bitPos / 8] & (1 << (bitPos % 8))) != 0;
      }

      // Dies ist die set-Accessormethode des Indexers.
      set
      {
         if ((bitPos < 0) || (bitPos >= m_numBits))
            throw new ArgumentOutOfRangeException("bitPos",
                 bitPos.ToString());

         if (value)
         {
            // Das indizierte Bit auf 1 setzen.
            m_byteArray[bitPos / 8] = (Byte)
               (m_byteArray[bitPos / 8] | (1 << (bitPos % 8)));
         }
         else
         {
            // Das indizierte Bit auf 0 setzen.
            m_byteArray[bitPos / 8] = (Byte)
               (m_byteArray[bitPos / 8] & ~(1 << (bitPos % 8)));
         }
      }
   }
}


///////////////////////////////////////////////////////////////////////////////


public sealed class Program {
   public static void Main() {
      // Ein BitArray fr 14 Bits anlegen.
      BitArray ba = new BitArray(14);

      // Jedes zweite Bit auf 1 setzen. Dazu rufen wir die set-Accessormethode auf.
      for (Int32 x = 0; x < 14; x++)
      {
         ba[x] = (x % 2 == 0);
      }

      // Zustand aller Bits anzeigen. Dazu rufen wir die get-Accessormethode auf.
      for (Int32 x = 0; x < 14; x++)
      {
         Console.WriteLine("Bit " + x + " ist " + (ba[x] ? "an" : "aus"));
      }
   }
}


//////////////////////////////// Dateiende //////////////////////////////////
