#define NonRobustThreadSafety

using System;


// Schritt 1: Definieren Sie einen Typ fr all die zustzlichen
// Informationen, die an die Empfnger des Ereignisses geschickt 
// werden sollen.
internal class NewMailEventArgs : EventArgs {

	private readonly String m_from, m_to, m_subject;

	public NewMailEventArgs(String from, String to, String subject) {
		m_from = from; m_to = to; m_subject = subject;
	}
		
	public String From    { get { return m_from;    } }
	public String To      { get { return m_to;      } }
	public String Subject { get { return m_subject; } }
}


///////////////////////////////////////////////////////////////////////////////


internal class MailManager {
#if NonRobustThreadSafety

   // Schritt 2: Definieren Sie das Ereignismember.
	public event EventHandler<NewMailEventArgs> NewMail;

#else

   // Privates Instanzfeld, das als Sperre fr die Threadsynchronisierung dient.
	private readonly Object m_eventLock = new Object();

   // Privates Feld, das auf den Kopf der Delegatenliste verweist.
	private EventHandler<NewMailEventArgs> m_NewMail;

   // Ereignismember zur Klasse hinzufgen
	public event EventHandler<NewMailEventArgs> NewMail {
      // Die 'add'-Methode wird explizit implementiert.
		add {
         // Die private Sperre beantragen, dann einen Handler
         // (bergeben als 'value') zur Delegatenliste hinzufgen.
			lock (m_eventLock) { m_NewMail += value; }
		}

      // Die 'remove'-Methode wird explizit implementiert.
		remove {
         // Die private Sperre beantragen, dann einen Handler
         // (bergeben als 'value') aus der Delegatenliste entfernen.
			lock (m_eventLock) { m_NewMail -= value; }
		}
	}

#endif

   // Schritt 3: Definieren Sie eine Methode, die das Ereignis
   // auslst, damit die registrierten Objekte benachrichtigt
   // werden, dass das Ereignis eingetreten ist.
   // Falls diese Klasse versiegelt ist, mssen Sie diese
   // Methode privat und nicht virtuell machen.
	protected virtual void OnNewMail(NewMailEventArgs e) {

#if NonRobustThreadSafety
      // Das Delegatfeld in einem temporren Feld zwischenspeichern, um
      // Threadsicherheit zu gewhrleisten.
		EventHandler<NewMailEventArgs> temp = NewMail;
#else
      // Das Delegatfeld in einem temporren Feld zwischenspeichern, um
      // Threadsicherheit zu gewhrleisten.
		EventHandler<NewMailEventArgs> temp = m_NewMail;
#endif

      // Falls irgendwelche Objekt sich als Empfnger fr unser
      // Ereignis angemeldet haben, werden sie benachrichtigt.
      if (temp != null) temp(this, e);
	}

   // Schritt 4: Definieren Sie eine Methode, die die Eingaben
   // in das gewnschte Ereignis umsetzt.
   public void SimulateNewMail(String from, String to, String subject) {

      // Ein Objekt anlegen, das die Informationen speichert, die wir
      // an die Empfnger unserer Benachrichtigung weitergeben wollen.
      NewMailEventArgs e = new NewMailEventArgs(from, to, subject);

      // Aufrufen unserer virtuellen Methode, um unser Objekt zu
      // benachrichtigen, dass das Ereignis eingetreten ist. Falls
      // diese Methode nicht berschrieben wurde, benachrichtigt
      // unser Objekt alle Methoden, die sich als Empfnger des
      // Ereignisses angemeldet haben.
      OnNewMail(e);
   }
}


///////////////////////////////////////////////////////////////////////////////


internal sealed class Fax {
   // Das MailManager-Objekt an den Konstruktor bergeben.
   public Fax(MailManager mm) {

      // Eine Instanz des EventHandler<NewMailEventArgs>-Delegaten anlegen,
      // die auf die Rckrufmethode FaxMsg verweist. Die Rckrufmethode
      // beim NewMail-Ereignis von MailManager anmelden.
		mm.NewMail += FaxMsg;
	}

   // Dies ist die Methode, die MailManager aufruft, wenn
   // eine neue E-Mail eingetroffen ist.
   private void FaxMsg(Object sender, NewMailEventArgs e) {

      // Der Parameter 'sender' verweist auf das MailManager-Objekt, falls
      // Sie mit dem Objekt kommunizieren wollen.

      // Der Parameter 'e' beschreibt die zustzlichen Informationen
      // ber das Ereignis, die der MailManager liefert.

      // Normalerweise wrden wir hier die E-Mail weiterfaxen. Diese
      // Testimplementierung zeigt die Informationen in der Konsole an.
      Console.WriteLine("Faxen der E-Mail:");
      Console.WriteLine("   Absender={0}, Empfnger={1}, Betreff={2}",
         e.From, e.To, e.Subject);
   }

   // Diese Methode knnte aufgerufen werden, um das Fax-Objekt wieder
   // als Empfnger fr das NewMail-Ereignis abzumelden. Es bekommt
   // dann keine Benachrichtigungen mehr.
   public void Unregister(MailManager mm)
   {

      // Beim NewMail-Ereignis von MailManager abmelden.
      mm.NewMail -= FaxMsg;
   }
}


///////////////////////////////////////////////////////////////////////////////


internal sealed class Pager {
   // Das MailManager-Objekt an den Konstruktor bergeben.
   public Pager(MailManager mm) {

      // Eine Instanz des ProcessMailMsgEventHandler-Delegaten anlegen,
      // die auf die Rckrufmethode SendMsgToPager verweist. Die Rckrufmethode
      // beim ProcessMailMsg-Ereignis von MailManager anmelden.
      mm.NewMail += SendMsgToPager;
   }

   // Dies ist die Methode, die MailManager aufruft, wenn
   // eine neue E-Mail eingetroffen ist.
   private void SendMsgToPager(Object sender, NewMailEventArgs e) {

      // Der Parameter 'sender' verweist auf das MailManager-Objekt, falls
      // Sie mit dem Objekt kommunizieren wollen.

      // Der Parameter 'e' beschreibt die zustzlichen Informationen
      // ber das Ereignis, die der MailManager liefert.

      // Normalerweise wrden wir hier die E-Mail an einen Pager schicken. Diese
      // Testimplementierung zeigt die Informationen in der Konsole an.
      Console.WriteLine("Senden der E-Mail an einen Pager:");
		Console.WriteLine("   Absender={0}, Empfnger={1}, Betreff={2}",
			e.From, e.To, e.Subject);
	}

	public void Unregister(MailManager mm) {

		// Eine Instanz des ProcessMailMsgEventHandler-Delegaten 
		// anlegen, die auf unsere Rckrufmethode FaxMsg verweist.
		//MailManager.ProcessMailMsgEventHandler callback = new MailManager.ProcessMailMsgEventHandler(FaxMsg);

      // Beim ProcessMailMsg-Ereignis von MailManager abmelden.
		mm.NewMail -= SendMsgToPager;
	}
}


///////////////////////////////////////////////////////////////////////////////


public sealed class Program {
   public static void Main() {
      // Ein MailManager-Objekt anlegen.
      MailManager mm = new MailManager();

      // Ein Fax-Objekt anlegen und ihm das MailManager-Objekt bergeben.
      Fax fax = new Fax(mm);

      // Ein Pager-Objekt anlegen und ihm das MailManager-Objekt bergeben.
      Pager pager = new Pager(mm);

      // Eingehen einer E-Mail simulieren.
      mm.SimulateNewMail("Jeffrey", "Kristin", "I Love You!");

      // Das Fax-Objekt zwingen, sich selbst beim MailManager abzumelden.
      fax.Unregister(mm);

      // Eingehen einer E-Mail simulieren.
      mm.SimulateNewMail("Jeffrey", "Mom & Dad", "Happy Birthday.");
   }
}


//////////////////////////////// Dateiende //////////////////////////////////
