using System;
using System.Collections.Generic;


///////////////////////////////////////////////////////////////////////////////


// Diese Klasse dient dazu, bei der Benutzung von EventSet etwas mehr 
// Typsicherheit und besser pflegbaren Code zur Verfgung zu stellen.
public sealed class EventKey : Object {
}


///////////////////////////////////////////////////////////////////////////////


public sealed class EventSet {
   // Das private Wrterbuch zum Verwalten der EventKey -> Delegate-Zuordnungen.
	private Dictionary<EventKey, Delegate> m_events =
		new Dictionary<EventKey, Delegate>();

   // Fgt eine EventKey -> Delegate-Zuordnung hinzu, falls sie noch nicht vorhanden ist,
	// oder kombiniert einen Delegaten mit einem vorhandenen EventKey.
	public void Add(EventKey eventKey, Delegate handler) {
		lock (m_events) {
			Delegate d;
			m_events.TryGetValue(eventKey, out d);
			m_events[eventKey] = Delegate.Combine(d, handler);
		}
	}

	// Entfernt einen Delegaten aus einem EventKey (sofern vorhanden) und 
	// entfernt die EventKey -> Delegate-Verknpfung, falls dies der letzte Delegat war.
	public void Remove(EventKey eventKey, Delegate handler) {
		lock (m_events) {
			// Keine Ausnahme auslsen, falls versucht wird, einen Delegaten
			// aus einem EventKey zu entfernen, der nicht vorhanden ist.
			Delegate d;
			if (m_events.TryGetValue(eventKey, out d)) {
				d = Delegate.Remove(d, handler);

				// Falls ein Delegat brig bleibt, den neuen Kopfeintrag 
            // anpassen, ansonsten den EventKey entfernen.
				if (d != null) m_events[eventKey] = d;
				else m_events.Remove(eventKey);
			}
		}
   }

   // Das Ereignis fr den angegebenen EventKey auslsen.
	public void Raise(EventKey eventKey, Object sender, EventArgs e) {
      // Keine Ausnahme auslsen, falls der EventKey nicht vorhanden ist.
      Delegate d;
		lock (m_events) { m_events.TryGetValue(eventKey, out d); }
		if (d != null) {
			// Weil das Wrterbuch mehrere unterschiedliche Delegattypen enthalten kann,
			// ist es unmglich, einen Aufruf des Delegaten zu konstruieren, der schon 
         // beim Kompilieren typsicher ist. Daher rufe ich ich die Methode DynamicInvoke 
         // des Typs System.Delegate auf und bergebe ihr die Argumente der Rckrufmethode
         // als Objektarray. Intern berprft DynamicInvoke, ob die Typsicherheit der Argumente
         // fr die Rckrufmethode erfllt ist, und ruft die Rckrufmethode dann auf.
			// Stimmen die Typen nicht berein, lst DynamicInvoke eine Ausnahme aus.
			d.DynamicInvoke(new Object[] { sender, e });
		}
   }
}


//////////////////////////////// Dateiende //////////////////////////////////
