/******************************************************************************
Module:  TypeWithLotsOfEvents.cs
Notices: Copyright (c) 2005 Jeffrey Richter
******************************************************************************/


using System;


///////////////////////////////////////////////////////////////////////////////


// Definition des von EventArgs abgeleiteten Typs fr dieses Ereignis.
public class FooEventArgs : EventArgs { }


// Definition des von EventArgs abgeleiteten Typs fr dieses Ereignis.
public class BarEventArgs : EventArgs { }


///////////////////////////////////////////////////////////////////////////////


internal class TypeWithLotsOfEvents {

   // Definieren eines privaten Instanzfelds, das auf eine Auflistung verweist.
   // Die Auflistung verwaltet einen Satz von Ereignis/Delegatpaaren.
   // HINWEIS: Der Typ EventSet ist nicht Teil der FCL, ich habe ihn selbst definiert.
   private readonly EventSet m_eventSet = new EventSet();

	// ber die geschtzte Eigenschaft haben abgeleitete Typen Zugriff auf die Auflistung.
	protected EventSet EventSet { get { return m_eventSet; } }

	#region Code to support the Foo event
   // 2. Definieren der Member fr das Ereignis Foo.
   // 2a. Anlegen eines statischen, schreibgeschtzten Objekts, das dieses Ereignis identifiziert.
   // Jedes Objekt hat einen anderen Hashcode, mit dem es die Delegatenliste dieses
   // Ereignisses in der Auflistung des Objekts abruft.
   protected static readonly EventKey s_fooEventKey = new EventKey();

   // 2b. Definieren der Accessormethoden des Ereignisses zum Hinzufgen/Entfernen
   // des Delegaten in der Auflistung.
   public event EventHandler<FooEventArgs> Foo {
      add    { m_eventSet.Add(s_fooEventKey, value); }
		remove { m_eventSet.Remove(s_fooEventKey, value); }
   }

   // 2c. Definieren der geschtzten, virtuellen On-Methode fr dieses Ereignis.
   protected virtual void OnFoo(FooEventArgs e) {
		m_eventSet.Raise(s_fooEventKey, this, e);
   }

   // 2d. Definieren der Methode, die Eingaben fr dieses Ereignis bersetzt.
   public void SimulateFoo() {
      OnFoo(new FooEventArgs());
	}
	#endregion

	#region Code to support the Bar event
   // 3. Definieren der Member fr das Ereignis Bar.
   // 3a. Anlegen eines statischen, schreibgeschtzten Objekts, das dieses Ereignis identifiziert.
   // Jedes Objekt hat einen anderen Hashcode, mit dem es die Delegatenliste dieses
   // Ereignisses in der Auflistung des Objekts abruft.
   protected static readonly EventKey s_barEventKey = new EventKey();

   // 3b. Definieren der Accessormethoden des Ereignisses zum Hinzufgen/Entfernen
   // des Delegaten in der Auflistung.
   public event EventHandler<BarEventArgs> Bar {
		add    { m_eventSet.Add(s_barEventKey, value); }
		remove { m_eventSet.Remove(s_barEventKey, value); }
   }

   // 3c. Definieren der geschtzten, virtuellen On-Methode fr dieses Ereignis.
   protected virtual void OnBar(BarEventArgs e) {
		m_eventSet.Raise(s_barEventKey, this, e);
   }

   // 3d. Definieren der Methode, die Eingaben fr dieses Ereignis bersetzt.
   public void SimulateBar() {
      OnBar(new BarEventArgs());
   }
	#endregion
}


///////////////////////////////////////////////////////////////////////////////


internal sealed class TypeWithEventInterest {
   public TypeWithEventInterest(TypeWithLotsOfEvents twle) {

      // Anmelden unserer beiden Rckrufmethoden fr die Ereignisse, die 
      // das TypeWithLotsOfEvents-Objekt anbietet.
      twle.Foo += FooEventCallback;
      twle.Bar += BarEventCallback;
   }

   // Wird aufgerufen, wenn das TypeWithLotsOfEvents-Objekt das Ereignis Foo auslst.
   private void FooEventCallback(Object sender, FooEventArgs e) {
      Console.WriteLine("Foo-Benachrichtigung erhalten");
   }

   // Wird aufgerufen, wenn das TypeWithLotsOfEvents-Objekt das Ereignis Bar auslst.
   private void BarEventCallback(Object sender, BarEventArgs e) {
      Console.WriteLine("Bar-Benachrichtigung erhalten");
   }

   public void UnregisterBar(TypeWithLotsOfEvents twle) {
      twle.Bar -= BarEventCallback;
   }
}


///////////////////////////////////////////////////////////////////////////////


public sealed class Program {
   public static void Main() {
      // Der folgende Code testet das Ereignis.
      TypeWithLotsOfEvents twle = new TypeWithLotsOfEvents();
      TypeWithEventInterest twei = new TypeWithEventInterest(twle);

      twle.SimulateFoo();
      twle.SimulateBar();
      twei.UnregisterBar(twle);

      twle.SimulateFoo();
      twle.SimulateBar();

		twei.UnregisterBar(twle);
		twle.SimulateFoo();
		twle.SimulateBar();

      Console.WriteLine("Drcken Sie die <Eingabetaste>, um diese Anwendung zu schlieen.");
      Console.ReadLine();
   }
}


//////////////////////////////// Dateiende //////////////////////////////////
