using System;


///////////////////////////////////////////////////////////////////////////////


public enum Color /* : byte */ {
   White,       // Bekommt den Wert 0
   Red,         // Bekommt den Wert 1
   Green,       // Bekommt den Wert 2
   Blue,        // Bekommt den Wert 3
   Orange       // Bekommt den Wert 4
}

public static class Program {
   public static void Main() {
      Console.WriteLine(Enum.GetUnderlyingType(typeof(Color)));

      // Variable vom Typ Color anlegen und mit Blue initialisieren.
      Color c = Color.Blue;
      Console.WriteLine(c);               // "Blue" (Allgemeines Format))
      Console.WriteLine(c.ToString());    // "Blue" (Allgemeines Format))
      Console.WriteLine(c.ToString("G")); // "Blue" (Allgemeines Format))
      Console.WriteLine(c.ToString("D")); // "3"    (Dezimalformat)
      Console.WriteLine(c.ToString("X")); // "03"   (Hexadezimalformat)

      // Die folgende Zeile zeigt "Blue" an.
      Console.WriteLine(Enum.Format(typeof(Color), 3, "G"));

      // Gibt ein Array mit allen in Color definierten Werten zurck.
      Color[] colors = (Color[])Enum.GetValues(typeof(Color));
      Console.WriteLine("Anzahl der definierten Symbole: " + colors.Length);
      Console.WriteLine("Wert\tSymbol\n-----\t------");
      foreach (Color color in colors)
      {
         // Jedes Symbol in den Formaten Dezimal und Allgemein anzeigen.
         Console.WriteLine("{0,5:D}\t{0:G}", color);
      }

      // Da Orange als 4 definiert ist, wird 'c' mit 4 initialisiert.
      c = (Color) Enum.Parse(typeof(Color), "orange", true);

      // Da Brown nicht definiert ist, wird die Ausnahme
      // ArgumentException ausgelst.
      try {
         c = (Color) Enum.Parse(typeof(Color), "Brown", false);
      }
      catch (ArgumentException) {
         Console.WriteLine("Brown ist im Enumerationstyp Color nicht definiert.");
      }

      // Legt eine Instanz des Enumerationstyps Color an, die den Wert 1 hat.
      c = (Color) Enum.Parse(typeof(Color), "1", false);

      // Legt eine Instanz des Enumerationstyps Color an, die den Wert 23 hat.
      c = (Color) Enum.Parse(typeof(Color), "23", false);

      // Zeigt "True" an, weil Color das Symbol Red als 1 definiert.
      Console.WriteLine(Enum.IsDefined(typeof(Color), 1));

      // Zeigt "True" an, weil Color das Symbol White als 0 definiert.
      Console.WriteLine(Enum.IsDefined(typeof(Color), "White"));

      // Zeigt "False" an, weil zwischen Gro- und Kleinschreibung unterschieden wird.
      Console.WriteLine(Enum.IsDefined(typeof(Color), "white"));

      // Zeigt "False" an, weil Color kein Symbol fr den Wert 10 definiert.
      Console.WriteLine(Enum.IsDefined(typeof(Color), 10));

      
      SetColor((Color) 3);  // Blue
      try {
         SetColor((Color) 547);  // Nicht in Color definiert
      }
      catch (ArgumentOutOfRangeException e) {
         Console.WriteLine(e);
      }
   }

   public static void SetColor(Color c) {
      if (!Enum.IsDefined(typeof(Color), c)) {
         throw(new ArgumentOutOfRangeException("c", c, "You didn't pass a valid Color"));
      }
      // Als Farbe die Werte White, Red, Green, Blue oder Orange einstellen.
   }
}