/******************************************************************************
Module:  Arrays.cs
Notices: Copyright (c) 2001 Jeffrey Richter
******************************************************************************/


using System;
using System.IO;


///////////////////////////////////////////////////////////////////////////////


public static class Program {
	public static void Main() {
		String[] sa = new String[1];
		Array a1 = Array.CreateInstance(typeof(String), new Int32[] { 1 }, new Int32[] { 0 });
		Array a2 = Array.CreateInstance(typeof(String), new Int32[] { 1 }, new Int32[] { 1 });
		Console.WriteLine(sa.GetType().ToString());
		Console.WriteLine(a1.GetType().ToString());
		Console.WriteLine(a2.GetType().ToString());

      Console.WriteLine("\n----- Array Demo -----");
      ArrayDemo();
      ArrayCasting();
   }


///////////////////////////////////////////////////////////////////////////////


   static void ArrayRankInfo(String name, Array a) {
      Console.WriteLine("Anzahl der Dimensionen im Array \"{0}\" (vom Typ {1}): ",
         name, a.GetType().ToString(), a.Rank);
      for (int r = 0; r < a.Rank; r++) {
         Console.WriteLine("Dimension: {0}, Startindex = {1},  Endindex = {2}",
            r, a.GetLowerBound(r), a.GetUpperBound(r));
      }
      Console.WriteLine();
   }

   static void ArrayDemo() {
      Int32[] arrayOfInt32s = new Int32[10];
      ArrayRankInfo("arrayOfInt32s", arrayOfInt32s);

      String[,] matrixOfStrings = new String[10, 20];
      ArrayRankInfo("matrixOfStrings", matrixOfStrings);

      // Ein Array mit drei Int32s anlegen: Startindex = -2, Endindex = 0
      Array arrayOfObjects =  Array.CreateInstance(typeof(Object), new Int32[] { 3 }, new Int32[] { -2 });
      ArrayRankInfo("arrayOfObjects", arrayOfObjects);

      arrayOfObjects.SetValue(10, -2);
      arrayOfObjects.SetValue(20, -1);
      arrayOfObjects.SetValue(30, +0);
      Console.WriteLine("Arrayelemente: a[-2] = {0}, a[-1] = {1}, a[0] = {2}", 
         arrayOfObjects.GetValue(-2), arrayOfObjects.GetValue(-1), arrayOfObjects.GetValue(0));
   }

   static void ArrayCasting() {
      // Zweidimensionales FileStream-Array anlegen.
      FileStream[,] fs2dim = new FileStream[5, 10];

      // Implizite Typkonvertierung in ein zweidimensionales Object-Array.
      Object[,] o2dim = fs2dim;

      // Keine Konvertierung von einem zweidimensionalen Array in ein eindimensionales Array mglich.
      // Compiler meldet Fehler CS0030: Eine Konvertierung vom Typ 'object[*,*]' in
      // 'System.IO.Stream[]' ist nicht mglich.
      //Stream[] s1dim = (Stream[])o2dim;

      // Explizite Typkonvertierung in ein zweidimensionales Stream-Array.
      Stream[,] s2dim = (Stream[,])o2dim;

      // Explizite Typkonvertierung in ein zweidimensionales String-Array.
      // Wird kompiliert, lst aber whrend der Laufzeit die Ausnahme
      // InvalidCastException aus.
      String[,] st2dim = (String[,])o2dim;

      // Eindimensionales Int32-Array (Werttypen) anlegen.
      Int32[] i1dim = new Int32[5];

      // Keine Konvertierung von einem Werttyparray zu etwas anderem mglich.
      // Compiler meldet Fehler CS0030: Eine Konvertierung vom Typ 'int[]' in
      // 'object[]' ist nicht mglich.
      //Object[] o1dim = (Object[])i1dim;

      // Neues Array anlegen, dann mit Array.Copy jedes Element
      // des Quellarrays in den gewnschten Typ im Zielarray umwandeln.
      // Der folgende Code erzeugt ein Array mit Verweisen auf
      // geschachtelte Int32s.
      Object[] ob1dim = new Object[i1dim.Length];
      Array.Copy(i1dim, ob1dim, i1dim.Length);
   } 

   static void DynamicArray() {
      // Ich mchte ein zweidimensionales Array [2005..2009][1..4].
      Int32[] lowerBounds = { 2005,  1 };
      Int32[] lengths     = {   5, 4 };
      Decimal[,] quarterlyRevenue = (Decimal[,]) 
         Array.CreateInstance(typeof(Decimal), lengths, lowerBounds);

      Int32 firstYear = quarterlyRevenue.GetLowerBound(0);
      Int32 lastYear  = quarterlyRevenue.GetUpperBound(0);
      Console.WriteLine("{0,4}  {1,9}  {2,9}  {3,9}  {4,9}", "Jahr", "Q1", "Q2", "Q3", "Q4");

      Random r = new Random();
      for (Int32 year = firstYear; year <= lastYear; year++) {
         Console.Write(year +"  ");

         for (Int32 quarter = quarterlyRevenue.GetLowerBound(1); quarter <= quarterlyRevenue.GetUpperBound(1); quarter++) {
            quarterlyRevenue[year, quarter] = r.Next(10000);

            Console.Write("{0,9:C}  ", quarterlyRevenue[year, quarter]);
         }
         Console.WriteLine();
      }
   } 
}


//////////////////////////////// Dateiende //////////////////////////////////
