using System;
using System.Diagnostics;

public static class Program
{
   public static void Main()
   {
      const Int32 numElements = 100;
      const Int32 testCount = 10000;
      Stopwatch sw;

      // Zweidimensionales Array deklarieren.
      Int32[,] a2Dim = new Int32[numElements, numElements];

      // Zweidimensionales Array als verzweigtes Array deklarieren (Vektor aus Vektoren).
      Int32[][] aJagged = new Int32[numElements][];
      for (Int32 x = 0; x < numElements; x++)
         aJagged[x] = new Int32[numElements];

      // 1: Zugriff auf alle Elemente des Arrays mit blicher sicherer Technik.
      sw = Stopwatch.StartNew();
      for (Int32 test = 0; test < testCount; test++)
         Safe2DimArrayAccess(a2Dim);
      Console.WriteLine("{0}: Safe2DimArrayAccess", sw.Elapsed);

      // 2: Zugriff auf alle Elemente des Arrays ber ein verzweigtes Array.
      sw = Stopwatch.StartNew();
      for (Int32 test = 0; test < testCount; test++)
         SafeJaggedArrayAccess(aJagged);
      Console.WriteLine("{0}: SafeJaggedArrayAccess", sw.Elapsed);

      // 3: Zugriff auf alle Elemente des Arrays mit unsicherer Technik.
      sw = Stopwatch.StartNew();
      for (Int32 test = 0; test < testCount; test++)
         Unsafe2DimArrayAccess(a2Dim);
      Console.WriteLine("{0}: Unsafe2DimArrayAccess", sw.Elapsed);
   }

   private static void Safe2DimArrayAccess(Int32[,] a)
   {
      for (Int32 x = 0; x < a.GetLength(0); x++)
      {
         for (Int32 y = 0; y < a.GetLength(1); y++)
         {
            Int32 element = a[x, y];
         }
      }
   }

   private static void SafeJaggedArrayAccess(Int32[][] a)
   {
      for (Int32 x = 0; x < a.GetLength(0); x++)
      {
         for (Int32 y = 0; y < a[x].GetLength(0); y++)
         {
            Int32 element = a[x][y];
         }
      }
   }

   private static unsafe void Unsafe2DimArrayAccess(Int32[,] a)
   {
      Int32 dim0LowIndex = 0;   // a.GetLowerBound(0);
      Int32 dim0HighIndex = a.GetUpperBound(0);

      Int32 dim1LowIndex = 0;   // a.GetLowerBound(1);
      Int32 dim1HighIndex = a.GetUpperBound(1);

      Int32 dim0Elements = dim0HighIndex - dim0LowIndex;

      fixed (Int32* pi = &a[0, 0])
      {
         for (Int32 x = dim0LowIndex; x <= dim0HighIndex; x++)
         {
            Int32 baseOfDim = x * dim0Elements;
            for (Int32 y = dim1LowIndex; y <= dim1HighIndex; y++)
            {
               Int32 element = pi[baseOfDim + y];
            }
         }
      }
   }
}
