using System;

// Diese Klasse implementiert die generische Schnittstelle IComparable<T> zweimal.
public sealed class Number : IComparable<Int32>, IComparable<String>
{
   private Int32 m_val = 5;

   // Diese Methode implementiert CompareTo von IComparable<Int32>.
   public Int32 CompareTo(Int32 n)
   {
      return m_val.CompareTo(n);
   }

   // Diese Methode implementiert CompareTo von IComparable<String>.
   public Int32 CompareTo(String s)
   {
      return m_val.CompareTo(Int32.Parse(s));
   }
}


public static class Program {
   public static void Main() {
      Number n = new Number();

      // Hier vergleiche ich den Wert in n mit einem Int32 (5).
      IComparable<Int32> cInt32 = n;
      Int32 result = cInt32.CompareTo(5);

      // Hier vergleiche ich den Wert in n mit einem String ("5").
      IComparable<String> cString = n;
      result = cString.CompareTo("5");
   }

   private static void SomeMethod1() {
      Int32 x = 1, y = 2;
      IComparable c = x;

      // CompareTo erwartet ein Object. Hier y (ein Int32) zu bergeben, ist okay.
      c.CompareTo(y);     // Hier findet Boxing statt.

      // CompareTo erwartet ein Object. Wird "2" (ein String) bergeben, wird der
      // Code kompiliert, aber whrend der Laufzeit wird eine ArgumentException ausgelst.
      c.CompareTo("2");
   }

   private static void SomeMethod2() {
      Int32 x = 1, y = 2;
      IComparable<Int32> c = x;

      // CompareTo erwartet einen Int32. Hier y (einen Int32) zu bergeben, ist okay.
      c.CompareTo(y);     // Hier findet Boxing statt.

      // CompareTo erwartet einen Int32.  Wird "2" (ein String) bergeben, meldet
      // der Compiler einen Fehler, dass String nicht in einen Int32 konvertiert
      // werden kann.
   }

   public sealed class SomeType {
      private static void Test() {
         Int32 x = 5;
         Guid g = new Guid();

         // Dieser Aufruf von M wird fehlerfrei kompiliert, weil
         // Int32 IComparable UND IConvertible implementiert.
         M(x);

         // Dieser Aufruf von M lst eine Fehlermeldung des Compilers aus, 
         // weil Guid IComparable implementiert, aber nicht IConvertible.
         // M(g);
      }

      // T, der Typparameter von M, ist so eingeschrnkt, dass er nur mit Typen arbeitet,
      // die SOWOHL die Schnittstelle IComparable ALS AUCH IConvertible implementieren.
      private static Int32 M<T>(T t) where T : IComparable, IConvertible
      {
         // ...
         return 0;
      }
   }
}
