#define Version2
using System;

#if Version1
internal class Base : IComparable {

   // Explizite Schnittstellenmethodenimplementierung
   Int32 IComparable.CompareTo(Object o) {
      Console.WriteLine("CompareTo von Base");
      return 0;
   }
}

internal class Derived : Base, IComparable {

   // ffentliche Methode, die auch die Schnittstellenimplementierung ist.
   public Int32 CompareTo(Object o) {
      Console.WriteLine("CompareTo von Derived");

      // Bei diesem Versuch, die explizite Schnittstellenmethodenimplementierung
      // der Basisklasse aufzurufen, meldet der Compiler einen Fehler:
      // Fehler CS0117: 'Base' enthlt keine Definition fr 'CompareTo'.
      base.CompareTo(o);
      return 0;
   }
}
#endif

#if Version2
internal class Base : IComparable {

   // Explizite Schnittstellenmethodenimplementierung
   Int32 IComparable.CompareTo(Object o) {
      Console.WriteLine("IComparable.CompareTo von Base");
      return CompareTo(o);   // Ruft jetzt die virtuelle Methode auf.
   }

   // Virtuelle Methode fr abgeleitete Klassen (diese Methode kann
   // einen beliebigen Namen tragen).
   public virtual Int32 CompareTo(Object o) {
      Console.WriteLine("Virtuelle CompareTo von Base");
      return 0;
   }
}

internal class Derived : Base, IComparable {

   // Eine ffentliche Methode, die auch die Schnittstellenimplementierung ist.
   public override Int32 CompareTo(Object o) {
      Console.WriteLine("CompareTo von Derived");

      // Jetzt knnen wir die virtuelle Methode von Base aufrufen.
      base.CompareTo(o);
      return 0;
   }
}
#endif

public static class Program {
   public static void Main() {
      Derived d = new Derived();
      d.CompareTo(null);
   }
}