using System;
using System.Text;

// Die Komponente Light (Lampe) definieren.
internal sealed class Light
{
   // Diese Methode gibt den Status der Lampe zurck.
   public String SwitchPosition()
   {
      return "Die Lampe ist aus";
   }
}

// Die Komponente Fan (Ventilator) definieren.
internal sealed class Fan
{
   // Diese Methode gibt den Status des Ventilators zurck.
   public String Speed()
   {
      throw new InvalidOperationException("Der Ventilator ist wegen berhitzung defekt");
   }
}

// Die Komponente Speaker (Lautsprecher) definieren.
internal sealed class Speaker
{
   // Diese Methode gibt den Status des Lautsprechers zurck.
   public String Volume()
   {
      return "Lautstrke ist hoch";
   }
}

public sealed class Program
{

   // Definition des Delegaten, mit dem der Status einer
   // Komponente ermittelt werden kann.
   private delegate String GetStatus();

   public static void Main()
   {
      // Eine leere Delegatenkette.
      GetStatus getStatus = null;

      // Die drei Komponenten anlegen und ihre Statusmethoden
      // zur Delegatenkette hinzufgen.
      getStatus += new GetStatus(new Light().SwitchPosition);
      getStatus += new GetStatus(new Fan().Speed);
      getStatus += new GetStatus(new Speaker().Volume);

      // Zusammengefassten Statusbericht anzeigen, der den Zustand
      // der drei Komponenten wiedergibt.
      Console.WriteLine(GetComponentStatusReport(getStatus));
   }

   // Diese Methode fragt den Status der verschiedenen Komponenten
   // ab und liefert einen Bericht zurck.
   private static String GetComponentStatusReport(GetStatus status)
   {

      // Falls die Kette leer ist, gibt es nichts zu tun.
      if (status == null) return null;

      // Hier stellen wir den Bericht zusammen.
      StringBuilder report = new StringBuilder();

      // Ein Array holen, bei dem jedes Element ein Delegat aus der Kette ist.
      Delegate[] arrayOfDelegates = status.GetInvocationList();

      // In einer Schleife ber alle Delegaten des Arrays gehen.
      foreach (GetStatus getStatus in arrayOfDelegates)
      {

         try
         {
            // Den Statusstring einer Komponente holen und an den Bericht anhngen.
            report.AppendFormat("{0}{1}{1}", getStatus(), Environment.NewLine);
         }
         catch (InvalidOperationException e)
         {
            // Fehlermeldung fr diese Komponente in den Bericht einfgen.
            Object component = getStatus.Target;
            report.AppendFormat(
               "Keinen Status von {1}{2} empfangen{0}  Fehler: {3}{0}{0}",
               Environment.NewLine,
               ((component == null) ? "" : component.GetType() + "."),
               getStatus.Method.Name,
               e.Message);
         }
      }

      // Den kompletten Bericht zurckliefern.
      return report.ToString();
   }
}
