using System;
using System.Threading;

internal sealed class AClass {
   private static void CallbackWithoutNewingADelegateObject() {
      ThreadPool.QueueUserWorkItem(
         delegate(Object obj) { Console.WriteLine(obj); }, 5);
   }
}

internal sealed class AClass2 {
   private static void UsingLocalVariablesInTheCallbackCode(Int32 numToDo) {

      // Einige lokale Variablen
      Int32[] squares = new Int32[numToDo];
      AutoResetEvent done = new AutoResetEvent(false);

      // Eine Reihe von Aufgaben mit anderen Threads durchfhren.
      for (Int32 n = 0; n < squares.Length; n++) {
         ThreadPool.QueueUserWorkItem(
            delegate(Object obj)
            {
               Int32 num = (Int32) obj;

               // Diese Aufgabe wrde normalerweise lnger dauern.
               squares[num] = num * num;

               // Falls dies die letzte Aufgabe ist, soll der Hauptthread weiterlaufen.
               if (Interlocked.Decrement(ref numToDo) == 0)
                  done.Set();
            }, n);
      }

      // Warten, bis alle anderen Threads beendet sind.
      done.WaitOne();

      // Ergebnisse anzeigen.
      for (Int32 n = 0; n < squares.Length; n++)
         Console.WriteLine("Index {0}, Square={1}", n, squares[n]);
   }
}


internal static class Program {
   static void Main() {
      // Stringarray erstellen und initialisieren.
      String[] names = { "Jeff", "Kristin", "Aidan" };

      // Nur die Namen holen, die ein kleines 'i' enthalten.
      Char charToFind = 'i';
      names = Array.FindAll(names, delegate(String name) 
         { return (name.IndexOf(charToFind) >= 0); });

      // Die Zeichen jedes Strings in Grobuchstaben konvertieren.
      names = Array.ConvertAll<String, String>(names, delegate(String name) 
         { return name.ToUpper(); });

      // Namen sortieren.
      Array.Sort(names, delegate(String name1, String name2) 
         { return String.Compare(name1, name2); });

      // Ergebnisse anzeigen.
      Array.ForEach(names, delegate(String name) { Console.WriteLine(name); });
   }
}