/******************************************************************************
Module:  DynamicDelegates.cs
Notices: Copyright (c) 2006 Jeffrey Richter
******************************************************************************/


using System;
using System.Reflection;
using System.IO;


///////////////////////////////////////////////////////////////////////////////


// Einige verschiedene Delegatdefinitionen
internal delegate Object TwoInt32s(Int32 n1, Int32 n2);
internal delegate Object OneString(String s1);


///////////////////////////////////////////////////////////////////////////////


public static class Program {
   public static void Main(String[] args) {
      if (args.Length < 2) {
         String fileName = Path.GetFileNameWithoutExtension(
             Assembly.GetEntryAssembly().Location);
      String usage =
         @"Aufruf:" +
         "{0}{1} delType methodName [Arg1] [Arg2]" +
         "{0}   dabei muss delType entweder TwoInt32s oder OneString sein" +
         "{0}   falls delType TwoInt32s ist, muss methodName Add oder Subtract sein" +
         "{0}   falls delType OneString ist, muss methodName NumChars oder Reverse sein" +
         "{0}" +
         "{0}Beispiele:" +
         "{0}   {1} TwoInt32s Add 123 321" +
         "{0}   {1} TwoInt32s Subtract 123 321" +
         "{0}   {1} OneString NumChars \"Halli Hallo\"" +
         "{0}   {1} OneString Reverse  \"Halli Hallo\"";
      Console.WriteLine(usage, Environment.NewLine, fileName);
         return;
      }

      // Parameter delType in einen Delegattyp konvertieren.
      Type delType = Type.GetType(args[0]);
      if (delType == null)
      {
         Console.WriteLine("Parameter delType ungltig: " + args[0]);
         return;
      }

      Delegate d;
      try
      {
         // Argument Arg1 in eine Methode konvertieren.
         MethodInfo mi = typeof(Program).GetMethod(args[1],
            BindingFlags.NonPublic | BindingFlags.Static);

         // Ein Delegatobjekt erstellen, das einen Wrapper fr die statische Methode bildet.
         d = Delegate.CreateDelegate(delType, mi);
      }
      catch (ArgumentException)
      {
         Console.WriteLine("Parameter methodName ungltig: " + args[1]);
         return;
      }

      // Ein Array erstellen, das nur die Argumente enthlt, die
      // ber das Delegatobjekt an die Methode bergeben werden sollen.
      Object[] callbackArgs = new Object[args.Length - 2];

      if (d.GetType() == typeof(TwoInt32s))
      {
         try
         {
            // Die String-Argumente in Int32-Argumente konvertieren.
            for (Int32 a = 2; a < args.Length; a++)
               callbackArgs[a - 2] = Int32.Parse(args[a]);
         }
         catch (FormatException)
         {
            Console.WriteLine("Argumente mssen Integer sein.");
            return;
         }
      }

      if (d.GetType() == typeof(OneString))
      {
         // Das String-Argument einfach kopieren.
         Array.Copy(args, 2, callbackArgs, 0, callbackArgs.Length);
      }

      try
      {
         // Den Delegaten aufrufen und Ergebnis anzeigen.
         Object result = d.DynamicInvoke(callbackArgs);
         Console.WriteLine("Ergebnis = " + result);
      }
      catch (TargetParameterCountException)
      {
         Console.WriteLine("Falsche Zahl von Argumenten angegeben.");
      }
   }

   // Diese Rckrufmethode hat 2 Int32-Parameter.
   private static Object Add(Int32 n1, Int32 n2)
   {
      return n1 + n2;
   }

   // Diese Rckrufmethode hat 2 Int32-Parameter.
   private static Object Subtract(Int32 n1, Int32 n2)
   {
      return n1 - n2;
   }

   // Diese Rckrufmethode hat einen String-Parameter.
   private static Object NumChars(String s1)
   {
      return s1.Length;
   }

   // Diese Rckrufmethode hat einen String-Parameter.
   private static Object Reverse(String s1)
   {
      Char[] chars = s1.ToCharArray();
      Array.Reverse(chars);
      return new String(chars);
   }
}


//////////////////////////////// Dateiende //////////////////////////////////
