using System;
using System.IO;
using System.Collections.Generic;

public static class Program {
   public static void Main() {
      Boolean b = ComparingGenericVariables.Op.M<ComparingGenericVariables.Op>(null, null);
   }

   private static Boolean MethodTakingAnyType<T>(T o) {
      T temp = o;
      Console.WriteLine(o.ToString());
      Boolean b = temp.Equals(o);
      return b;
   }

   private static T Min<T>(T o1, T o2) where T : IComparable<T> {
      if (o1.CompareTo(o2) < 0) return o1;
      return o2;
   }

   private static void CallMin() {
      Object o1 = "Jeff", o2 = "Richter";
      //Object oMin = Min<Object>(o1, o2);	// Fehler CS0309
   }
}

internal sealed class OverloadingByArity {
   // Es ist erlaubt, die folgenden Typen zu definieren:
   internal sealed class AType { }
   internal sealed class AType<T> { }
   internal sealed class AType<T1, T2> { }

   // Fehler: Konflikt mit AType<T>, das keine Einschrnkungen hat.
   //internal sealed class AType<T> where T : IComparable<T> { }

   // Fehler: Konflikt mit AType<T1, T2>
   //internal sealed class AType<T3, T4> { }

   internal sealed class AnotherType {
      // Es ist erlaubt, die folgenden Methoden zu definieren:
      private static void M() { }
      private static void M<T>() { }
      private static void M<T1, T2>() { }

      // Fehler: Konflikt mit M<T>, das keine Einschrnkungen hat.
      //private static void M<T>() where T : IComparable<T> { }

      // Fehler: Konflikt mit M<T1, T2>
      //private static void M<T3, T4>() { }
   }
}

internal static class OverridingVirtualGenericMethod {
   internal class Base {
      public virtual void M<T1, T2>()
         where T1 : struct
         where T2 : class {
      }
   }

   internal sealed class Derived : Base {
      public override void M<T3, T4>()
         /*where T3 : struct 
         where T4 : class  */
      {
      }
   }
}

internal static class PrimaryConstraints {
   internal static class PrimaryConstraintOfStream<T> where T : Stream {
      public static void M(T stream) {
         stream.Close();   // OK
      }
   }

   internal static class PrimaryConstraintOfClass<T> where T : class {
      public static void M() {
         T temp = null;	// Erlaubt, weil T ein Verweistyp sein muss.
      }
   }

   internal static class PrimaryConstraintOfStruct<T> where T : struct {
      public static T Factory() {
         // Erlaubt, weil alle Werttypen implizit einen
         // ffentlichen, parameterlosen Konstruktor haben.
         return new T();
      }
   }
}

internal static class SecondaryConstraints {
   private static List<TBase> ConvertIList<T, TBase>(IList<T> list)
      where T : TBase {

      List<TBase> baseList = new List<TBase>(list.Count);
      for (Int32 index = 0; index < list.Count; index++) {
         baseList.Add(list[index]);
      }
      return baseList;
   }

   private static void CallingConvertIList() {
      // Ein List<String> anlegen und initialisieren (implementiert IList<String>).
      IList<String> ls = new List<String>();
      ls.Add("A String");

      // Das IList<String> in ein IList<Object> konvertieren.
      IList<Object> lo = ConvertIList<String, Object>(ls);

      // Das IList<String> in ein IList<IComparable> konvertieren.
      IList<IComparable> lc = ConvertIList<String, IComparable>(ls);

      // Das IList<String> in ein IList<IComparable<String>> konvertieren.
      IList<IComparable<String>> lcs =
         ConvertIList<String, IComparable<String>>(ls);

      // Das IList<String> in ein IList<Exception> konvertieren.
      //IList<Exception> le = ConvertIList<String, Exception>(ls);	// Fehler
   }
}

internal sealed class ConstructorConstraints {
   internal sealed class ConstructorConstraint<T> where T : new() {
      public static T Factory() {
         // Erlaubt, weil alle Werttypen implizit einen ffentlichen,
         // parameterlosen Konstruktor haben und die Einschrnkung
         // sicherstellt, dass auch jeder angegebene Verweistyp einen
         // ffentlichen, parameterlosen Konstruktor hat.
         return new T();
      }
   }
}

internal sealed class CastingAGenericTypeVariable {
   private void CastingAGenericTypeVariable1<T>(T obj) {
      //Int32 x = (Int32)obj;    // Fehler
      //String s = (String)obj;  // Fehler
   }
   private void CastingAGenericTypeVariable2<T>(T obj) {
      Int32 x = (Int32)(Object)obj;    // Kein Fehler
      String s = (String)(Object)obj;  // Kein Fehler
   }
   private void CastingAGenericTypeVariable3<T>(T obj) {
      String s = obj as String;  // Kein Fehler
   }
}

internal sealed class SettingAGenericTypeVariableToADefaultValue {
   private void SettingAGenericTypeVariableToNull<T>() {
      //T temp = null;	// Fehler, ein Werttyp kann nicht auf null gesetzt werden.
   }

   private void SettingAGenericTypeVariableToDefaultValue<T>() {
      T temp = default(T);	// OK
   }
}

internal sealed class ComparingGenericVariables {
   private void ComparingAGenericTypeVariableWithNull<T>(T obj) {
      if (obj == null) { /* Wird bei einem Werttyp niemals ausgefhrt */ }
   }
   private void ComparingTwoGenericTypeVariables<T>(T o1, T o2) {
      //if (o1 == o2) { }	// Fehler
   }

   internal class Op {
      public static Boolean operator ==(Op o1, Op o2) { return true; }
      public static Boolean operator !=(Op o1, Op o2) { return false; }
      public static Boolean M<T>(T o1, T o2) where T : Op {
         return o1 == o2;
      }
   }
}

#if false
internal static class UsingGenericTypeVariablesAsOperands {
   private T Sum<T>(T num) where T : struct {
      T sum = default(T);
      for (T n = default(T); n < num; n++)
         sum += n;
      return sum;
   }
}
#endif