using System;
using System.Diagnostics;
using System.Reflection;
using System.Collections.Generic;


[assembly: CLSCompliant(true)]


[Serializable]
[DefaultMemberAttribute("Main")]
[DebuggerDisplayAttribute("Richter", Name="Jeff", Target=typeof(Program))]
public sealed class Program {
   [Conditional("Debug")]
   [Conditional("Release")]
   public void DoSomething() { }

   public Program() {
   }

   [CLSCompliant(true)]
   [STAThread]
   public static void Main() {
      // Die Attribute anzeigen, die diesem Typ zugeordnet sind.
      ShowAttributes(typeof(Program));

      // Die Methoden ermitteln, die fr den Typ definiert sind.
      MemberInfo[] members = typeof(Program).FindMembers(
         MemberTypes.Constructor | MemberTypes.Method,
         BindingFlags.DeclaredOnly | BindingFlags.Instance |
         BindingFlags.Public | BindingFlags.Static,
         Type.FilterName, "*");

      foreach (MemberInfo member in members) {
         // Die Attribute anzeigen, die diesem Member zugeordnet sind.
         ShowAttributes(member);
      }
   }

   private static void ShowAttributes(MemberInfo attributeTarget) {
      IList<CustomAttributeData> attributes = 
         CustomAttributeData.GetCustomAttributes(attributeTarget);

      Console.WriteLine("{0} zugeordnete Attribute: {1}",
         attributeTarget.Name, (attributes.Length == 0 ? "Keine" : String.Empty));

      foreach (CustomAttributeData attribute in attributes) {
         // Den Typ jedes angewendeten Attributs anzeigen.
         Type t = attribute.Constructor.DeclaringType;
         Console.WriteLine("  {0}", t.ToString());
         Console.WriteLine("    Aufgerufener Konstructor={0}", attribute.Constructor);

         IList<CustomAttributeTypedArgument> posArgs = attribute.ConstructorArguments;
         Console.WriteLine("    Positionelle Argumente, die an den Konstruktor bergeben werden:" +
            ((posArgs.Count == 0) ? " Keine" : String.Empty));
         foreach (CustomAttributeTypedArgument pa in posArgs) {
             Console.WriteLine("      Type={0}, Value={1}", pa.ArgumentType, pa.Value);
         }


         IList<CustomAttributeNamedArgument> namedArgs = attribute.NamedArguments;
         Console.WriteLine("    Benannte Argumente, die nach dem Anlegen gesetzt werden:" +
            ((namedArgs.Count == 0) ? " Keine" : String.Empty));
         foreach(CustomAttributeNamedArgument na in namedArgs) {
            Console.WriteLine("     Name={0}, Type={1}, Value={2}", 
               na.MemberInfo.Name, na.TypedValue.ArgumentType, na.TypedValue.Value);
         }

         Console.WriteLine();
      }
      Console.WriteLine();
   }
}
