using System;


[Flags]
public enum Accounts {
   Savings   = 0x0001,
   Checking  = 0x0002,
   Brokerage = 0x0004
}


[AttributeUsage(AttributeTargets.Class)]
public sealed class AccountsAttribute : Attribute {
   private Accounts m_accounts;

   public AccountsAttribute(Accounts accounts) {
     m_accounts = accounts;
   }


   public override Boolean Match(Object obj)
   {
      // Falls die Basisklasse Match implementiert und die
      // Basisklasse nicht Attribute ist, knnen Sie die
      // Kommentarzeichen in der folgenden Zeile entfernen.
      // if (!base.Match(obj)) return false;

      // Da this nicht null ist, knnen die Objekte nicht
      // bereinstimmen, falls obj gleich null ist.
      // HINWEIS: Sie knnen diese Zeile lschen, wenn Sie sicher
      // sind, dass der Basistyp Match korrekt implementiert.
      if (obj == null) return false;

      // Falls die Objekte unterschiedliche Typen haben, knnen
      // sie nicht bereinstimmen.
      // HINWEIS: Sie knnen diese Zeile lschen, wenn Sie sicher
      // sind, dass der Basistyp Match korrekt implementiert.
      if (this.GetType() != obj.GetType()) return false;

      // obj in unseren Typ konvertieren, um auf die Felder zuzugreifen.
      // HINWEIS: Diese Typkonvertierung kann nicht fehlschlagen, weil
      // wir bereits wissen, dass die Objekte denselben Typ haben.
      AccountsAttribute other = (AccountsAttribute)obj;

      // Die Felder vergleichen.
      // Dieses Beispiel prft, ob accounts in this eine Untermenge
      // von accounts in 'other' ist.
      if ((other.m_accounts & m_accounts) != m_accounts)
         return false;

      return true;   // Objekte stimmen berein.
   }


   public override Boolean Equals(Object obj)
   {
      // Falls die Basisklasse Equals implementiert und die
      // Basisklasse nicht Object ist, knnen Sie die
      // Kommentarzeichen in der folgenden Zeile entfernen.
      // if (!base.Equals(obj)) return false;

      // Da this nicht null ist, knnen die Objekte nicht gleich
      // sein, falls obj gleich null ist.
      // HINWEIS: Sie knnen diese Zeile lschen, wenn Sie sicher
      // sind, dass der Basistyp Equals korrekt implementiert.
      if (obj == null) return false;

      // Falls die Objekte unterschiedliche Typen haben, knnen
      // sie nicht gleich sein.
      // HINWEIS: Sie knnen diese Zeile lschen, wenn Sie sicher
      // sind, dass der Basistyp Equals korrekt implementiert.
      if (this.GetType() != obj.GetType()) return false;

      // obj in unseren Typ konvertieren, um auf die Felder zuzugreifen.
      // HINWEIS: Diese Typkonvertierung kann nicht fehlschlagen, weil
      // wir bereits wissen, dass die Objekte denselben Typ haben.
      AccountsAttribute other = (AccountsAttribute)obj;

      // Felder vergleichen und feststellen, ob sie denselben Wert
      // haben.
      // Dieses Beispiel prft, ob accounts in this den
      // accounts in 'other' entspricht.
      if (other.m_accounts != m_accounts)
         return false;

      return true;   // Objekte sind gleich.
   }

   // GetHashCode berschreiben, weil wir Equals berschreiben.
   public override Int32 GetHashCode()
   {
      return (Int32)m_accounts;
   }
}


[Accounts(Accounts.Savings)]
internal sealed class ChildAccount { }


[Accounts(Accounts.Savings | Accounts.Checking | Accounts.Brokerage)]
internal sealed class AdultAccount { }


public sealed class Program {
   public static void Main() {
      CanWriteCheck(new ChildAccount());
      CanWriteCheck(new AdultAccount());

      // Diese Anweisung soll lediglich demonstrieren, dass die
      // Methode korrekt mit einem Typ funktioniert, dem keine
      // AccountsAttribute-Attribute zugeordnet sind.
      CanWriteCheck(new Program());
   }

   private static void CanWriteCheck(Object obj) {
      // Eine Instanz des Attributtyps anlegen und mit den Werten
      // initialisieren, nach denen wir explizit suchen.
      Attribute checking = new AccountsAttribute(Accounts.Checking);

      // Die Attributinstanz anlegen, die dem Typ zugeordnet wurde.
      Attribute validAccounts = Attribute.GetCustomAttribute(
         obj.GetType(), typeof(AccountsAttribute), false);

      // Falls das Attribut dem Typ zugeordnet wurde UND im Attribut
      // der Wert "Checking" gesetzt ist, kann der Typ einen
      // Scheck ausstellen.
      if ((validAccounts != null) && checking.Match(validAccounts))
      {
         Console.WriteLine("{0}-Typen drfen Schecks ausstellen.", obj.GetType());
      }
      else
      {
         Console.WriteLine("{0}-Typen drfen KEINE Schecks ausstellen.", obj.GetType());
      }
   }
}