using System;

public static class Program {
   public static void Main() {
      UsingNullable();
      ConversionsAndCasting();
      Operators();
      NullableCodeSize(5, 10);
      NullCoalescingOperator();
      Boxing();
      Unboxing();
      
      UnboxingAllocations();
      GetType();
      InterfaceMethods();
   }
   
   private static void UsingNullable() {
      Nullable<Int32> x = 5;
      Nullable<Int32> y = null;
      Console.WriteLine("x: HasValue={0}, Value={1}", 
         x.HasValue, x.Value);
      Console.WriteLine("y: HasValue={0}, Value={1}", 
         y.HasValue, y.GetValueOrDefault());
   }

   private static void ConversionsAndCasting() {
      // Implizite Konvertierung von normalem Int32 zu Nullable<Int32>
      Int32? a = 5;

      // Implizite Konvertierung von 'null' zu Nullable<Int32>
      Int32? b = null;

      // Explizite Konvertierung von Nullable<Int32> zu normalem Int32
      Int32 c = (Int32)a;

      // Typumwandlung zwischen primitiven Nullable-Typen
      Double? d = 5; // Int32->Double?  (d ist 5.0 als double)
      Double? e = b; // Int32?->Double? (e ist null)
   }

   private static void Operators() {
      Int32? a = 5;
      Int32? b = null;

      // Unre Operatoren (+  ++  -  --  !  ~)
      a++;    // a = 6
      b = -b; // b = null

      // Binre Operatoren (+  -  *  /  %  &  |  ^  <<  >>)
      a = a + 3;  // a = 9
      b = b * 3;  // b = null;

      // Gleichheitsoperatoren (==  !=)
      if (a == null) { /* Nein  */ } else { /* Ja */ }
      if (b == null) { /* Ja */ } else { /* Nein  */ }
      if (a != b) { /* Ja */ } else { /* Nein  */ }

      // Vergleichsoperatoren (<  >  <=  >=)
      if (a < b) { /* Nein  */ } else { /* Ja */ }
   }

   private static Int32? NullableCodeSize(Int32? a, Int32? b) {
      return a + b;
   }

   private static void NullCoalescingOperator() {
      Int32? b = null;

      // Die folgende Zeile ist quivalent zu:
      // x = (b.HasValue) ? b.Value : 123
      Int32 x = b ?? 123;
      Console.WriteLine(x);  // "123"

      // Die folgende Zeile ist quivalent zu:
      // String temp = GetFilename();
      // filename = (temp != null) ? temp : "Unbenannt";
      String filename = GetFilename() ?? "Unbenannt";
   }

   private static String GetFilename() { return null; }
   private static void Boxing() {
      // Boxing von Nullable<T> ergibt null oder ein geschachteltes T
      Int32? n = null;
      Object o = n;  // o ist null
      Console.WriteLine("o ist null={0}", o == null);  // "True"

      n = 5;
      o = n;   // o verweist auf ein geschachteltes Int32.
      Console.WriteLine("Typ von o={0}", o.GetType()); // "System.Int32"
   }

   private static void Unboxing() {
      // Geschachteltes Int32 anlegen.
      Object o = 5;

      // In ein Nullable<Int32> und ein Int32 entschachteln.
      Int32? a = (Int32?)o;  // a = 5
      Int32 b = (Int32)o;  // b = 5

      // Einen Verweis erstellen, der mit null initialisiert ist.
      o = null;

      // Versuchen, den Verweis in ein Nullable<Int32> und ein Int32 zu "entschachteln".
      a = (Int32?)o;       // a = null
      b = (Int32)o;       // NullReferenceException
   }

   private static void UnboxingAllocations() {
      const Int32 count = 1000000;

      // Ein geschachteltes Int32 anlegen.
      Object o = 5;

      Int32 numGCs = GC.CollectionCount(0);
      for (Int32 x = 0; x < count; x++)
      {
         Int32 unboxed = (Int32)o;
      }
      Console.WriteLine("Anzahl der Garbage Collection-Vorgnge={0}", GC.CollectionCount(0) - numGCs);

      numGCs = GC.CollectionCount(0);
      for (Int32 x = 0; x < count; x++)
      {
         Int32? unboxed = (Int32?)o;
      }
      Console.WriteLine("Anzahl der Garbage Collection-Vorgnge={0}", GC.CollectionCount(0) - numGCs);
   }

   private static void GetType() {
      Int32? x = 5;

      // Die folgende Zeile zeigt "System.Int32" an, nicht "System.Nullable<Int32>".
      Console.WriteLine(x.GetType());
   }
   private static void InterfaceMethods() {
      Int32? n = 5;
      Int32 result = ((IComparable<Int32>)n).CompareTo(5); // Lsst sich kompilieren und ausfhren.
      Console.WriteLine(result);                           // 0
      result = ((IComparable)(Int32)n).CompareTo(5);       // Mhsam
      Console.WriteLine(result);                           // 0
   }
}