using System;
using System.Diagnostics;
using System.Threading;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Collections;
using System.Windows.Forms;


public sealed class Program {
   public static void Main() {
      Console.Write("Soll der Handler fr unbehandelte Ausnahmen installiert werden? (J/N)");
      ConsoleKeyInfo key = Console.ReadKey(false);
      Console.WriteLine();
      Console.Write("Handler fr unbehandelte Ausnahmen installiert = ");
      if (Char.ToUpper(key.KeyChar) == 'J') {
         // Unsere Rckrufmethode MgdUEFilter bei der AppDomain registrieren, damit
		   // sie aufgerufen wird, wenn eine unbehandelte Ausnahme auftritt.
		   AppDomain.CurrentDomain.UnhandledException += 
			   new UnhandledExceptionEventHandler(MgdUEFilter);
         Console.WriteLine(true);
      } else Console.WriteLine(false);

      while (true) {
         Console.WriteLine("Welche Art von unbehandelter Ausnahme wollen Sie erzwingen?");
         Console.Write("1=Main, 2=ThreadPool, 3=Manual, 4=Finalizer, 5=Native, 6=WindowsFormWindowProc?");
         key = Console.ReadKey(false);
         Console.WriteLine();

         switch (key.KeyChar) {
            case '1':
               Throw("Main");
               break;

            case '2':
               ThreadPool.QueueUserWorkItem(delegate { Throw("ThreadPool"); });
               break;

            case '3':
               Thread t = new Thread((ThreadStart) delegate { Throw("Manual"); });
               t.Start();
               break;

            case '4':
               new Program();
               GC.Collect();
               break;

            case '5':
               new NativeThread();
               break;

            case '6':
               Application.Run(new MyForm());
               break;
         }
         Console.ReadLine();
      }
   }

   private static void Throw(String threadType) {
      throw new InvalidOperationException(
         String.Format("Erzwungene Ausnahme in Thread {0}.", threadType));
   }

   ~Program() {
      Throw("Finalize");
   }

   private sealed class MyForm : Form {
      protected override void OnPaint(PaintEventArgs e) {
         Throw("Windows Form Window Procedure");
      }
   }


   private static void MgdUEFilter(Object sender, UnhandledExceptionEventArgs e) {
      // Dieser String enthlt die Meldungen, die angezeigt oder protokolliert werden.
      String info;
      Console.WriteLine("MgdUEFilter");

      // Initialisieren des Strings.
      Exception ex = e.ExceptionObject as Exception;
      if (ex != null) {
         // Eine unbehandelte, CLS-kompatible Ausnahme wurde ausgelst.
         // Gehen Sie nach Belieben vor: Sie knnen auf die Felder von Exception zugreifen
         // (Message, StackTrace, HelpLink, InnerException usw.)
         info = ex.ToString();
      } else {
         // Eine unbehandelte, nicht CLS-kompatible Ausnahme wurde ausgelst.
         // Gehen Sie nach Belieben vor: Sie knnen nur die Methoden aufrufen, die 
         // von Object definiert werden (ToString, GetType usw.)
         info = String.Format("Nicht CLS-kompatible Ausnahme: Typ={0}, String={1}",
            e.ExceptionObject.GetType(), e.ExceptionObject.ToString());
      }

#if DEBUG
      // Bei DEBUG-Builds der Anwendung wird der Debugger gestartet, damit
      // wir den Fehler untersuchen und korrigieren knnen.
      if (!e.IsTerminating) {
         // Unbehandelte Ausnahme in einem Threadpool- oder Finalizerthread.
         Debugger.Launch();
      } else {
         // Unbehandelte Ausnahme in einem verwalteten Thread.
         // Standardmig verknpft die CLR automatisch einen Debugger, aber
         // wir knnen das auch mit der folgenden Zeile erzwingen:
         Debugger.Launch();
      }
#else
      // Bei RELEASE-Builds der Anwendung wird die Ausnahme angezeigt oder protokolliert, 
		// damit der Benutzer sie an den Entwickler melden kann.
		if (!e.IsTerminating) {
         // Unbehandelte Ausnahme in einem Threadpool- oder Finalizerthread.
         // Bei Threadpool- oder Finalizerthreads ist es meist sinnvoll, die Ausnahme einfach
		   // zu protokollieren, aber nicht anzuzeigen. Das mssen Sie aber fr jede 
		   // Anwendung individuell entscheiden.
		} else {
         // Unbehandelte Ausnahme in einem verwalteten Thread.
			// Die CLR beendet gleich die Anwendung. Sie sollten die Ausnahme
		   // anzeigen und/oder protokollieren.
		}
#endif

      Console.WriteLine("Unbehandelte Ausnahme abgefangen:");
      Console.WriteLine(e.ExceptionObject);
      Console.WriteLine("IsTerminating: " + e.IsTerminating);
      Console.ReadLine();
      //if (!Debugger.IsAttached) Debugger.Launch();
   }
}

internal sealed class NativeThread {
   public static void SetUnhandledExceptionFilter() {
      SetUnhandledExceptionFilter(TLEF);
   }

   private delegate ExceptionFilterDisposition TopLevelExceptionFilter(UIntPtr exceptionPointers);
   private enum ExceptionFilterDisposition {
      ExecuteHandler = 1,
      ContinueSearch = 0,
      ContinueExecution = -1
   }
   [DllImport("Kernel32")]
   private static extern UIntPtr SetUnhandledExceptionFilter(TopLevelExceptionFilter filter);
   private static ExceptionFilterDisposition TLEF(UIntPtr ep) {
      Console.WriteLine("TopLevelExceptionFilter");
      return ExceptionFilterDisposition.ContinueSearch;
   }


   public NativeThread() {
      UInt32 threadId;
      IntPtr hThread = CreateThread(IntPtr.Zero, 0, 
         new ThreadStartRoutine(UnmgdThreadFunc), UIntPtr.Zero, 0, out threadId);
      CloseHandle(hThread);
   }

   public delegate void ThreadStartRoutine(UIntPtr ThreadParameter);

   [DllImport("Kernel32")]
   public static extern IntPtr CreateThread(IntPtr SecurityAttributes, UInt32 StackSize,
      ThreadStartRoutine StartFunction, UIntPtr ThreadParameter, 
      UInt32 CreationFlags, out UInt32 ThreadId);

   [DllImport("Kernel32")]
   private static extern Boolean CloseHandle(IntPtr handle);

   private static void UnmgdThreadFunc(UIntPtr p) {
      Console.WriteLine("In UnmgdThreadFunc");
      Int32 x = 0;
      x = 10 / x;
   }
}

