#define Version2
using System;
using System.IO;


internal sealed class GCBeep {
   // Dies ist die Finalize-Methode.
   ~GCBeep() {
      // Die Klasse wird abgeschlossen. Pieeep!
      Console.Beep();

      // Falls die AppDomain nicht beendet wird und der Prozess
      // nicht beendet wird, ein neues Objekt anlegen, das bei 
      // der nchsten Garbage Collection abgeschlossen wird.
      if (!AppDomain.CurrentDomain.IsFinalizingForUnload() &&
          !Environment.HasShutdownStarted)
         new GCBeep();
   }
}

public sealed class Program {
   public static void Main() {
      // Ein einzelnes GCBeep-Objekt anlegen. Dieses Objekt piepst jedes Mal,
      // wenn eine Garbage Collection stattfindet.
      new GCBeep();

      // Eine ganze Menge 100-Byte-Objekte anlegen.
      for (Int32 x = 0; x < 10000; x++)
      {
         Console.WriteLine(x);
         Byte[] b = new Byte[100];
      }
   }
}

internal sealed class TempFile {
   private String m_filename = null;
   private FileStream m_fs;

#if Version1
   public TempFile(String filename) {
      // Die folgende Zeile knnte eine Ausnahme auslsen.
      m_fs = new FileStream(filename, FileMode.Create);

      // Den Namen der Datei speichern.
      m_filename = filename;
   }

   // Dies ist die Finalize-Methode.
   ~TempFile() {
      // Die korrekte Vorgehensweise ist, zu berprfen, ob der Dateiname null ist.
      // Schlielich knnen Sie nicht sicher sein, dass filename im Konstruktor
      // initialisiert wurde.
      if (m_filename != null)
         File.Delete(m_filename);
   }
#endif

#if Version2
   public TempFile(String filename) {
      try {
         // Die folgende Zeile knnte eine Ausnahme auslsen.
         m_fs = new FileStream(filename, FileMode.Create);

         // Den Namen der Datei speichern.
         m_filename = filename;
      }
      catch {
         // Falls irgendetwas schief geht, muss der Garbage Collector wissen,
         // dass er die Finalize-Methode nicht aufrufen darf. Die Methode
         // SuppressFinalize werde ich weiter unten vorstellen.
         GC.SuppressFinalize(this);

         // Den Aufrufer informieren, dass etwas schief gelaufen ist.
         throw;
      }
   }

   // Dies ist die Finalize-Methode.
   ~TempFile()
   {
      // Keine if-Anweisung ntig, weil dieser Code nur dann
      // ausgefhrt wird, wenn der Konstruktor erfolgreich war.
      File.Delete(m_filename);
   }
#endif
}
