using System;
using System.Runtime.InteropServices;

public static class Program
{
   public static void Main()
   {
      // Wird 0 bergeben, werden selten Garbage Collections ausgefhrt.
      MemoryPressureDemo(0);

      // Wird 10 MByte bergeben, werden oft Garbage Collections ausgefhrt.
      MemoryPressureDemo(10 * 1024 * 1024);

      // Handles verwalten
      HandleCollectorDemo();
   }

   private static void MemoryPressureDemo(Int32 size)
   {
      Console.WriteLine();
      Console.WriteLine("MemoryPressureDemo, size={0}", size);
      // Eine Reihe von Objekten anlegen. Wir geben ihre logische Gre an.
      for (Int32 count = 0; count < 15; count++)
      {
         new BigNativeResource(size);
      }

      // Fr Demonstrationszwecke erzwingen, dass alles aufgerumt wird.
      GC.Collect();
      GC.WaitForPendingFinalizers();
   }

   private sealed class BigNativeResource
   {
      private Int32 m_size;

      public BigNativeResource(Int32 size)
      {
         m_size = size;
         if (m_size > 0)
         {
            // Wir spiegeln dem Garbage Collector vor, dass das Objekt
            // in Wirklichkeit grer ist.
            GC.AddMemoryPressure(m_size);
         }
         Console.WriteLine("BigNativeResource erstellt.");
      }

      ~BigNativeResource()
      {
         if (m_size > 0)
         {
            // Wir spiegeln dem Garbage Collector vor, dass das Objekt
            // mehr Speicher freigegeben hat.
            GC.RemoveMemoryPressure(m_size);
         }
         Console.WriteLine("BigNativeResource zerstrt.");
      }
   }

   private static void HandleCollectorDemo()
   {
      Console.WriteLine();
      Console.WriteLine("HandleCollectorDemo");
      for (Int32 count = 0; count < 10; count++)
      {
         new LimitedResource();
      }

      // Fr Demonstrationszwecke erzwingen, dass alles aufgerumt wird.
      GC.Collect();
      GC.WaitForPendingFinalizers();
   }

   private sealed class LimitedResource
   {
      // Einen HandleCollector erstellen und ihm mitteilen, dass Garbage Collections
      // ausgefhrt werden sollen, falls zwei oder mehr dieser Objekte auf dem Heap liegen.
      private static HandleCollector s_hc = new HandleCollector("LimitedResource", 2);

      public LimitedResource()
      {
         // Wir teilen dem HandleCollector mit, dass ein weiteres
         // LimitedResource-Objekt zum Heap hinzugefgt wurde.
         s_hc.Add();
         Console.WriteLine("LimitedResource erstellt. Count={0}", s_hc.Count);
      }
      ~LimitedResource()
      {
         // Wir teilen dem HandleCollector mit, dass ein
         // LimitedResource-Objekt vom Heap entfernt wurde.
         s_hc.Remove();
         Console.WriteLine("LimitedResource zerstrt. Count={0}", s_hc.Count);
      }
   }
}
