using System;
using System.Text;
using System.Reflection;
using System.Collections.Generic;

public static class Program
{
   public static void Main()
   {
      // Die Assemblys, die wir mit Reflektion untersuchen wollen, explizit laden.
      LoadAssemblies();

      // Zhler und Liste der Ausnahmetypen initialisieren.
      Int32 totalPublicTypes = 0, totalExceptionTypes = 0;
      List<String> exceptionTree = new List<String>();

      // Durch alle Assemblys gehen, die in diese AppDomain geladen sind.
      foreach (Assembly a in AppDomain.CurrentDomain.GetAssemblies())
      {

         // Durch alle Typen gehen, die in dieser Assembly definiert sind.
         foreach (Type t in a.GetExportedTypes())
         {
            totalPublicTypes++;

            // Typ ignorieren, falls er keine ffentliche Klasse ist.
            if (!t.IsClass || !t.IsPublic) continue;

            // Einen String mit der Ableitungshierarchie des Typs erstellen.
            StringBuilder typeHierarchy = new StringBuilder(t.FullName, 5000);

            // Grundannahme lautet, dass der Typ nicht von Exception abgeleitet ist.
            Boolean derivedFromException = false;

            // Prfen, ob System.Exception ein Basistyp dieses Typs ist.
            Type baseType = t.BaseType;
            while ((baseType != null) && !derivedFromException)
            {
               // Basistyp an das Ende des Strings anhngen.
               typeHierarchy.Append("-" + baseType);

               derivedFromException = (baseType == typeof(System.Exception));
               baseType = baseType.BaseType;
            }

            // Keine Basistypen mehr, nicht von Exception abgeleitet.
            // Jetzt den nchsten Typ prfen.
            if (!derivedFromException) continue;

            // Wir haben einen Typ gefunden, der von Exception abgeleitet ist.
            totalExceptionTypes++;

            // Bei diesem von Exception abgeleiteten Typ drehen wir
            // die Reihenfolge der Typen in der Hierarchie um.
            String[] h = typeHierarchy.ToString().Split('-');
            Array.Reverse(h);

            // Einen neuen String erstellen, bei dem die Hierarchie diese Reihenfolge hat:
            // Exception -> von Exception abgeleiteter Typ
            // Den String zur Liste der Ausnahmetypen hinzufgen.
            exceptionTree.Add(String.Join("-", h, 1, h.Length - 1));
         }
      }

      // Die Ausnahmetypen anhand der Hierarchie sortieren.
      exceptionTree.Sort();

      // Die Strukturdarstellung anzeigen.
      foreach (String s in exceptionTree)
      {
         // Bei diesem Typ trennen wir den Basistyp ab.
         String[] x = s.Split('-');

         // Abhngig von der Zahl der Basistypen einrcken und
         // den Typ anzeigen, der am weitesten abgeleitet ist.
         Console.WriteLine(new String(' ', 3 * x.Length) + x[x.Length - 1]);
      }

      // Statistik zu den untersuchten Typen anzeigen.
      Console.WriteLine("\n---> Von {0} Typen sind {1} " +
         "von System.Exception abgeleitet.",
         totalPublicTypes, totalExceptionTypes);
   }

   private static void LoadAssemblies()
   {
      String[] assemblies = {
         "System,                    PublicKeyToken={0}",
         "System.Data,               PublicKeyToken={0}",
         "System.Design,             PublicKeyToken={1}",
         "System.DirectoryServices,  PublicKeyToken={1}",
         "System.Drawing,            PublicKeyToken={1}",
         "System.Drawing.Design,     PublicKeyToken={1}",
         "System.Management,         PublicKeyToken={1}",
         "System.Messaging,          PublicKeyToken={1}",
         "System.Runtime.Remoting,   PublicKeyToken={0}",
         "System.Security,           PublicKeyToken={1}",
         "System.ServiceProcess,     PublicKeyToken={1}",
         "System.Web,                PublicKeyToken={1}",
         "System.Web.RegularExpressions, PublicKeyToken={1}",
         "System.Web.Services,       PublicKeyToken={1}",
         "System.Windows.Forms,      PublicKeyToken={0}",
         "System.Xml,                PublicKeyToken={0}",
      };

      String EcmaPublicKeyToken = "b77a5c561934e089";
      String MSPublicKeyToken = "b03f5f7f11d50a3a";

      // Version der Assembly ermitteln, die System.Object enthlt.
      // Wir nehmen an, dass die anderen Assemblys dieselbe Version haben.
      Version version =
         typeof(System.Object).Assembly.GetName().Version;

      // Die Assemblies, die wir ber Reflektion untersuchen wollen, explizit laden.
      foreach (String a in assemblies)
      {
         String AssemblyIdentity =
            String.Format(a, EcmaPublicKeyToken, MSPublicKeyToken) +
            ", Culture=neutral, Version=" + version;

         Assembly.Load(AssemblyIdentity);
      }
   }
}
