using System;
using System.IO;
using System.Reflection;
using System.Collections.Generic;
using Wintellect.HostSDK;

public static class Program
{
   public static void Main()
   {
      // Das Verzeichnis suchen, in dem die ausfhrbare Datei des Hosts liegt.
      String AddInDir = Path.GetDirectoryName(
         Assembly.GetEntryAssembly().Location);

      // Wir nehmen an, dass die Add-In-Assemblys im selben Verzeichnis
      // liegen wie die EXE-Datei des Hosts.
      String[] AddInAssemblies = Directory.GetFiles(AddInDir, "*.dll");

      // Eine Auflistung der verfgbaren Add-In-Typen erstellen.
      List<Type> AddInTypes = new List<Type>();

      // Die Add-In-Assemblys laden. Ermitteln, welche Typen der Host benutzen kann.
      foreach (String file in AddInAssemblies)
      {
         Assembly AddInAssembly = Assembly.LoadFrom(file);

         // Alle ffentlich exportierten Typen untersuchen.
         foreach (Type t in AddInAssembly.GetExportedTypes())
         {
            // Falls der Typ eine Klasse ist, die die Schnittstelle IAddIn
            // implementiert, kann der Host diesen Typ benutzen.
            if (t.IsClass && typeof(IAddIn).IsAssignableFrom(t))
            {
               AddInTypes.Add(t);
            }
         }
      }

      // Initialisierung fertig: Der Host hat die verfgbaren Add-Ins erkannt.

      // So kann der Host Add-In-Objekte anlegen und benutzen.
      foreach (Type t in AddInTypes)
      {
         IAddIn ai = (IAddIn)Activator.CreateInstance(t);
         Console.WriteLine(ai.DoSomething(5));
      }
   }
}
