using System;
using System.Reflection;

// Zwei Schnittstellen zum Testen definieren.
internal interface IBookRetailer : IDisposable {
   void Purchase();
   void ApplyDiscount();
}

internal interface IMusicRetailer {
   void Purchase();
}


// Diese Klasse implementiert zwei Schnittstellen, die in dieser Assembly
// definiert sind, und eine Schnittstelle, die in einer anderen Assembly
// definiert ist.
internal sealed class MyRetailer : IBookRetailer, IMusicRetailer, IDisposable
{
   // Methoden von IBookRetailer
   void IBookRetailer.Purchase() { }
   public void ApplyDiscount() { }

   // Methoden von IMusicRetailer
   void IMusicRetailer.Purchase() { }

   // Methoden von IDisposable
   public void Dispose() { }

   // Methoden von MyRetailer (keine Schnittstellenmethode)
   public void Purchase() { }
}


public static class Program
{
   public static void Main()
   {
      // Die Schnittstellen suchen, die von MyRetailer implementiert werden und
      // die in dieser Assembly definiert sind. Dazu verwenden wir einen Delegaten
      // fr eine Filtermethode, den wir anlegen und an FindInterfaces bergeben.
      Type t = typeof(MyRetailer);
      Type[] interfaces = t.FindInterfaces(TypeFilter,
         typeof(Program).Assembly);
      Console.WriteLine("MyRetailer implementiert die folgenden " +
         "Schnittstellen (definiert in dieser Assembly):");

      // Informationen ber jede Schnittstelle anzeigen.
      foreach (Type i in interfaces)
      {
         Console.WriteLine("\nSchnittstelle: " + i);

         // Die Typmethoden holen, die den Schnittstellenmethoden entsprechen.
         InterfaceMapping map = t.GetInterfaceMap(i);

         for (Int32 m = 0; m < map.InterfaceMethods.Length; m++)
         {
            // Anzeigen, welchen Namen die Schnittstellenmethode hat und
            // welche Typmethode diese Schnittstellenmethode implementiert.
            Console.WriteLine("   {0} wird implementiert von {1}",
               map.InterfaceMethods[m], map.TargetMethods[m]);
         }
      }
   }

   // Gibt true zurck, falls die Filterbedingung erfllt ist.
   private static Boolean TypeFilter(Type t, Object filterCriteria)
   {
      // true zurckgeben, falls die Schnittstelle in der Assembly definiert
      // ist, die in filterCriteria beschrieben ist.
      return t.Assembly == filterCriteria;
   }
}
