using System;
using System.Reflection;
using System.Collections.Generic;


public sealed class Program
{
   private const BindingFlags c_bf = BindingFlags.FlattenHierarchy |
      BindingFlags.Instance | BindingFlags.Static |
      BindingFlags.Public | BindingFlags.NonPublic;

   public static void Main()
   {
      // Gre des Heaps anzeigen, bevor wir irgendeine Reflektion ausfhren.
      Show("Bevor wir irgendetwas tun.");

      // Cache aus MethodInfo-Objekten fr alle Methoden in MSCorlib.dll anlegen.
      List<MethodBase> methodInfos = new List<MethodBase>();
      foreach (Type t in typeof(Object).Assembly.GetExportedTypes())
      {
         // Wir ignorieren generische Typen.
         if (t.IsGenericTypeDefinition) continue;

         MethodBase[] mb = t.GetMethods(c_bf);
         methodInfos.AddRange(mb);
      }

      // Anzahl der Methoden und Gre des Heaps anzeigen, nachdem
      // alle Methoden gebunden wurden.
      Console.WriteLine("Anzahl der Methoden={0:###,###}", methodInfos.Count);
      Show("Nach den Anlegen des Caches fr MethodInfo-Objekte.");

      // Cache aus RuntimeMethodHandles fr alle MethodInfo-Objekte anlegen.
      List<RuntimeMethodHandle> methodHandles =
         methodInfos.ConvertAll<RuntimeMethodHandle>(
            delegate(MethodBase mb) { return mb.MethodHandle; });

      Show("Caches fr MethodInfo und RuntimeMethodHandle gespeichert.");
      GC.KeepAlive(methodInfos); // Verhindern, dass der Cache gleich
      // vom Garbage Collector beseitigt wird.

      methodInfos = null;        // Jetzt darf der Garbage Collector den Cache beseitigen.
      Show("MethodInfo-Objekte wurden freigegeben.");

      methodInfos = methodHandles.ConvertAll<MethodBase>(
         delegate(RuntimeMethodHandle rmh)
         {
            return MethodBase.GetMethodFromHandle(rmh);
         });
      Show("Gre des Heaps nach dem Neuanlegen der MethodInfo-Objekte.");
      GC.KeepAlive(methodHandles);  // Verhindern, dass der Cache gleich
      // vom Garbage Collector beseitigt wird.
      GC.KeepAlive(methodInfos);    // Verhindern, dass der Cache gleich
      // vom Garbage Collector beseitigt wird.

      methodHandles = null;         // Jetzt darf der Garbage Collector den Cache beseitigen.
      methodInfos = null;           // Jetzt darf der Garbage Collector den Cache beseitigen.
      Show("MethodInfos und RuntimeMethodHandles wurden freigegeben.");
   }

   private static void Show(String s)
   {
      Console.WriteLine("Heapgre={0,12:##,###,###} - {1}",
         GC.GetTotalMemory(true), s);
   }
}
