using System;
using System.IO;
using System.Threading;

public static class Program {
   public static void Main() {
      //ReadMultipleFiles(@"C:\Boot.ini", @"c:\Point.cs");
      // Die Datei fr asynchrone E/A ffnen.
      FileStream fs = new FileStream(@"C:\Boot.ini", FileMode.Open,
         FileAccess.Read, FileShare.Read, 1024,
         FileOptions.Asynchronous);

      Byte[] data = new Byte[100];

      // Eine asynchrone Leseoperation mit dem FileStream beginnen.
      IAsyncResult ar = fs.BeginRead(data, 0, data.Length, null, null);

      // Hier irgendwelchen anderen Code ausfhren...

      // Diesen Thread anhalten, bis die asynchrone
      // Operation abgeschlossen ist und wir das Ergebnis erhalten.
      Int32 bytesRead = fs.EndRead(ar);

      // Keine anderen Operationen ntig, Datei schlieen.
      fs.Close();

      // Jetzt knnen wir auf das Bytearray zugreifen und das Ergebnis anzeigen.
      Console.WriteLine("Anzahl der gelesenen Bytes={0}", bytesRead);
      Console.WriteLine(BitConverter.ToString(data, 0, bytesRead));
   }

   private static void ReadMultipleFiles(params String[] pathnames)
   {
      AsyncStreamRead[] asrs = new AsyncStreamRead[pathnames.Length];

      for (Int32 n = 0; n < pathnames.Length; n++)
      {
         // Die Datei fr asynchrone E/A ffnen.
         Stream stream = new FileStream(pathnames[n], FileMode.Open,
            FileAccess.Read, FileShare.Read, 1024,
            FileOptions.Asynchronous);

         // Eine asynchrone Leseoperation mit dem Stream einleiten.
         asrs[n] = new AsyncStreamRead(stream, 100);
      }

      // Alle Streams wurden geffnet und alle Leseanforderungen wurden in die
      // Warteschlange gestellt. Sie laufen jetzt parallel!

      // Jetzt holen wir die Ergebnisse und zeigen sie an.
      for (Int32 n = 0; n < asrs.Length; n++)
      {
         Byte[] bytesRead = asrs[n].EndRead();

         // Jetzt knnen wir auf das Bytearray zugreifen und das Ergebnis anzeigen.
         Console.WriteLine("Anzahl der gelesenen Bytes={0}", bytesRead.Length);
         Console.WriteLine(BitConverter.ToString(bytesRead));
      }
   }

   private sealed class AsyncStreamRead
   {
      private Stream m_stream;
      private IAsyncResult m_ar;
      private Byte[] m_data;

      public AsyncStreamRead(Stream stream, Int32 numBytes)
      {
         m_stream = stream;
         m_data = new Byte[numBytes];

         // Eine asynchrone Leseoperation mit dem Stream einleiten.
         m_ar = stream.BeginRead(m_data, 0, numBytes, null, null);
      }

      public Byte[] EndRead()
      {
         // Diesen Thread anhalten, bis die asynchrone Operation
         // abgeschlossen ist und wir das Ergebnis erhalten.
         Int32 numBytesRead = m_stream.EndRead(m_ar);

         // Keine anderen Operationen ntig. Den Stream schlieen.
         m_stream.Close();

         // Gre des Arrays ndern, um Speicherplatz zu sparen.
         Array.Resize(ref m_data, numBytesRead);

         // Die Bytes zurckgeben.
         return m_data;
      }
   }
}
