using System;
using System.IO;
using System.Threading;

public static class Program {
   public static void Main() {
      PollingWithIsCompleted();
      PollingWithAsyncWaitHandle();
   }

   public static void PollingWithIsCompleted()
   {
      // Die Datei fr asynchrone E/A ffnen.
      FileStream fs = new FileStream(@"C:\Boot.ini", FileMode.Open,
         FileAccess.Read, FileShare.Read, 1024,
         FileOptions.Asynchronous);

      Byte[] data = new Byte[100];

      // Eine asynchrone Leseoperation mit dem FileStream einleiten.
      IAsyncResult ar = fs.BeginRead(data, 0, data.Length, null, null);

      while (!ar.IsCompleted)
      {
         Console.WriteLine("Operation nicht abgeschlossen. Es wird noch gewartet.");
         Thread.Sleep(10);
      }

      // Das Ergebnis holen. Hinweis: EndRead hlt diesen Thread NICHT an.
      Int32 bytesRead = fs.EndRead(ar);

      // Keine anderen Operationen ntig, Datei schlieen.
      fs.Close();

      // Jetzt knnen wir auf das Bytearray zugreifen und das Ergebnis anzeigen.
      Console.WriteLine("Anzahl der gelesenen Bytes={0}", bytesRead);
      Console.WriteLine(BitConverter.ToString(data, 0, bytesRead));
   }

   public static void PollingWithAsyncWaitHandle()
   {
      // Die Datei fr asynchrone E/A ffnen.
      FileStream fs = new FileStream(@"C:\Boot.ini", FileMode.Open,
         FileAccess.Read, FileShare.Read, 1024,
         FileOptions.Asynchronous);

      Byte[] data = new Byte[100];

      // Eine asynchrone Leseoperation mit dem FileStream einleiten.
      IAsyncResult ar = fs.BeginRead(data, 0, data.Length, null, null);

      while (!ar.AsyncWaitHandle.WaitOne(10, false))
      {
         Console.WriteLine("Operation nicht abgeschlossen. Es wird noch gewartet.");
      }

      // Das Ergebnis holen. Hinweis: EndRead hlt diesen Thread NICHT an.
      Int32 bytesRead = fs.EndRead(ar);

      // Keine anderen Operationen ntig, Datei schlieen.
      fs.Close();

      // Jetzt knnen wir auf das Bytearray zugreifen und das Ergebnis anzeigen.
      Console.WriteLine("Anzahl der gelesenen Bytes={0}", bytesRead);
      Console.WriteLine(BitConverter.ToString(data, 0, bytesRead));
   }
}
