using System;
using System.IO;
using System.Threading;

public static class Program {
   // Das Array ist statisch, damit Main und ReadIsDone darauf zugreifen knnen.
   private static Byte[] s_data = new Byte[100];

   public static void Main() {
      ReadMultipleFiles(@"C:\boot.ini", @"C:\point.cs");
      APMCallbackUsingAnonymousMethod();
      // Die ID des Thread anzeigen, der Main ausfhrt.
      Console.WriteLine("Hauptthread ID={0}",
         Thread.CurrentThread.ManagedThreadId);

      // Die Datei fr asynchrone E/A ffnen.
      FileStream fs = new FileStream(@"C:\Boot.ini", FileMode.Open,
         FileAccess.Read, FileShare.Read, 1024,
         FileOptions.Asynchronous);

      // Eine asynchrone Leseoperation mit dem FileStream einleiten.
      // Den FileStream (fs) an die Rckrufmethode (ReadIsDone) bergeben.
      fs.BeginRead(s_data, 0, s_data.Length, ReadIsDone, fs);

      // Hier wre es ntzlich, irgendwelchen anderen Code auszufhren...

      // Fr dieses Beispiel halte ich den primren Thread einfach an.
      Console.ReadLine();
   }

   private static void ReadIsDone(IAsyncResult ar)
   {
      // Die ID des Threads anzeigen, der ReadIsDone ausfhrt.
      Console.WriteLine("ReadIsDone-Thread ID={0}",
         Thread.CurrentThread.ManagedThreadId);

      // FileStream (state) aus dem IAsyncResult-Objekt extrahieren.
      FileStream fs = (FileStream)ar.AsyncState;

      // Das Ergebnis holen.
      Int32 bytesRead = fs.EndRead(ar);

      // Keine anderen Operationen ntig, Datei schlieen.
      fs.Close();

      // Jetzt knnen wir auf das Bytearray zugreifen und das Ergebnis anzeigen.
      Console.WriteLine("Anzahl der gelesenen Bytes={0}", bytesRead);
      Console.WriteLine(BitConverter.ToString(s_data, 0, bytesRead));
   }

   private static void APMCallbackUsingAnonymousMethod() {
      // Die ID des Threads anzeigen, der Main ausfhrt.
      Console.WriteLine("Hauptthread ID={0}",
         Thread.CurrentThread.ManagedThreadId);

      // Die Datei fr asynchrone E/A ffnen.
      FileStream fs = new FileStream(@"C:\Boot.ini", FileMode.Open,
         FileAccess.Read, FileShare.Read, 1024,
         FileOptions.Asynchronous);

      Byte[] data = new Byte[100];

      // Eine asynchrone Leseoperation mit dem FileStream einleiten.
      // Den FileStream (fs) an die Rckrufmethode (ReadIsDone) bergeben.
      fs.BeginRead(data, 0, data.Length,
         delegate(IAsyncResult ar)
         {
            // Die ID des Threads anzeigen, der ReadIsDone ausfhrt.
            Console.WriteLine("ReadIsDone-Thread ID={0}",
               Thread.CurrentThread.ManagedThreadId);

            // Das Ergebnis holen.
            Int32 bytesRead = fs.EndRead(ar);

            // Keine anderen Operationen ntig, Datei schlieen.
            fs.Close();

            // Jetzt knnen wir auf das Bytearray zugreifen und das Ergebnis anzeigen.
            Console.WriteLine("Anzahl der gelesenen Bytes={0}", bytesRead);
            Console.WriteLine(BitConverter.ToString(data, 0, bytesRead));

         }, null);

      // Hier wre es ntzlich, irgendwelchen anderen Code auszufhren...

      // Fr dieses Beispiel halte ich den primren Thread einfach an.
      Console.ReadLine();
   }

   private static void ReadMultipleFiles(params String[] pathnames)
   {
      for (Int32 n = 0; n < pathnames.Length; n++)
      {
         // Die Datei fr asynchrone E/A ffnen.
         Stream stream = new FileStream(pathnames[n], FileMode.Open,
            FileAccess.Read, FileShare.Read, 1024,
            FileOptions.Asynchronous);

         // Eine asynchrone Leseoperation mit dem Stream einleiten.
         new AsyncStreamRead(stream, 100,
            delegate(Byte[] data)
            {
               // Daten verarbeiten.
               Console.WriteLine("Anzahl der gelesenen Bytes={0}", data.Length);
               Console.WriteLine(BitConverter.ToString(data));
            });
      }

      // Alle Streams wurden geffnet und alle Leseanforderungen wurden in die
      // Warteschlange gestellt. Sie laufen jetzt parallel und werden verarbeitet,
      // nachdem sie abgeschlossen sind!

      // Der primre Thread knnte hier andere Arbeiten durchfhren, wenn er will...

      // Fr dieses Beispiel halte ich den primren Thread einfach an.
      Console.WriteLine("Drcken Sie die <Eingabetaste>, um das Programm zu beenden...");
      Console.ReadLine();
   }

   private delegate void StreamBytesRead(Byte[] streamData);

   private sealed class AsyncStreamRead
   {
      private Stream m_stream;
      private Byte[] m_data;
      StreamBytesRead m_callback;

      public AsyncStreamRead(Stream stream, Int32 numBytes,
         StreamBytesRead callback)
      {
         m_stream = stream;
         m_data = new Byte[numBytes];
         m_callback = callback;

         // Eine asynchrone Leseoperation mit dem Stream einleiten.
         stream.BeginRead(m_data, 0, numBytes, ReadIsDone, null);
      }

      // Wird aufgerufen, nachdem die E/A-Operation abgeschlossen ist.
      private void ReadIsDone(IAsyncResult ar)
      {
         Int32 numBytesRead = m_stream.EndRead(ar);

         // Keine anderen Operationen ntig. Den Stream schlieen.
         m_stream.Close();

         // Gre des Arrays ndern, um Speicherplatz zu sparen.
         Array.Resize(ref m_data, numBytesRead);

         // Rckrufmethode der Anwendung aufrufen.
         m_callback(m_data);
      }
   }
}
