#define SumIsDoneWithException
using System;
using System.Threading;

public static class Program {
   public static void Main() {
      // Eine Delegatvariable initialisieren, die auf eine Methode
      // verweist, die wir asynchron aufrufen wollen.
      SumDelegate sumDelegate = Sum;

#if !SumIsDoneWithException
      sumDelegate.BeginInvoke(1000000000, SumIsDone, sumDelegate);
#else
      sumDelegate.BeginInvoke(10000000000, SumIsDone, sumDelegate);
#endif

      // Hier wre es ntzlich, irgendwelchen anderen Code auszufhren...

      // Fr dieses Beispiel halte ich den primren Thread einfach an.
      Console.ReadLine();
   }

#if !SumIsDoneWithException
   private static void SumIsDone(IAsyncResult ar) {
      // Den SumDelegate (state) aus dem IAsyncResult-Objekt extrahieren.
      SumDelegate sumDelegate = (SumDelegate) ar.AsyncState;

      // Das Ergebnis holen.
      UInt64 sum = sumDelegate.EndInvoke(ar);

      // Das Ergebnis anzeigen.
      Console.WriteLine("Sum={0}", sum);
   }
#else
   private static void SumIsDone(IAsyncResult ar)
   {
      // Den SumDelegate (state) aus dem IAsyncResult-Objekt extrahieren.
      SumDelegate sumDelegate = (SumDelegate)ar.AsyncState;

      try
      {
         // Das Ergebnis holen; dabei knnte eine OverflowException ausgelst werden.
         UInt64 sum = sumDelegate.EndInvoke(ar);

         // Das Ergebnis anzeigen.
         Console.WriteLine("Sum={0}", sum);
      }
      catch (OverflowException)
      {
         // EndInvoke hat eine OverflowException ausgelst. Konsistenten Zustand herstellen.
         Console.WriteLine("Sum kann nicht angezeigt werden. Zahl ist zu gro.");
      }
   }
#endif

   private delegate UInt64 SumDelegate(UInt64 n);

   private static UInt64 Sum(UInt64 n) {
      UInt64 sum = 0;
      for (UInt64 i = 0; i <= n; i++) {
         checked {
            // Ich benutze checked, damit eine OverflowException ausgelst wird,
            // falls die Summe nicht in ein UInt64 passt.
            sum += i;
         }
      }
      return sum;
   }
}
