using System;
using System.IO;
using System.Security;
using System.Threading;
using System.Security.Permissions;

public static class Program
{
   public static void Main()
   {
      // Zeigen, dass die Methode einwandfrei funktioniert.
      AttemptAccess("Standardkontext");

      // Ausfhrungskontext des Threads ndern, sodass
      // alle Dateizugriffe verboten sind.
      new FileIOPermission(PermissionState.Unrestricted).Deny();

      // Zeigen, dass die Methode nicht mehr funktioniert, weil
      // der Ausfhrungskontext verndert wurde.
      AttemptAccess("Keine Dateiberechtigungen");

      ECFlowing();
      ECFlowingSuppressed();

      // Den aktuellen Ausfhrungskontext dieses Threads aufzeichnen.
      ExecutionContext ec = ExecutionContext.Capture();

      // Ausfhrungskontext des Threads ndern, sodass
      // Dateizugriffe erlaubt sind.
      SecurityPermission.RevertDeny();

      // Zeigen, dass die Methode einwandfrei funktioniert.
      AttemptAccess("Wieder im Standardkontext");

      ECCaptureAndRun(ec);
   }

   private static void ECFlowing()
   {
      // Eine Delegatvariable initialisieren, die auf eine Methode
      // verweist, die wir asynchron aufrufen wollen.
      WaitCallback wc = AttemptAccess;

      // Einen Thread aus dem Threadpool benutzen, um den Zugriff zu versuchen.
      // EndInvoke kehrt zurck, wenn der Thread in den Threadpool zurckkehrt.
      wc.EndInvoke(wc.BeginInvoke("ECFlowing", null, null));
   }

   private static void ECFlowingSuppressed()
   {
      // Eine Delegatvariable initialisieren, die auf eine Methode
      // verweist, die wir asynchron aufrufen wollen.
      WaitCallback wc = AttemptAccess;

      // Die CLR anweisen, den Ausfhrungskontext dieses Threads
      // vorerst nicht mehr in die Hilfsthreads zu kopieren.
      using (AsyncFlowControl afc = ExecutionContext.SuppressFlow())
      {
         wc.EndInvoke(wc.BeginInvoke("ECFlowingSuppressed", null, null));
      }
   }

   private static void ECCaptureAndRun(ExecutionContext ec)
   {
      // AttemptAccess aufrufen und dabei den vorher
      // aufgezeichneten Ausfhrungskontext benutzen.
      ExecutionContext.Run(ec, AttemptAccess,
         "ECCaptureAndRun mit Run");
   }

   private static void AttemptAccess(Object test)
   {
      // Grundannahme: Diese Methode schlgt fehl.
      Boolean success = false;
      try
      {
         // Versuchen, die Dateiattribute zu ermitteln.
         File.GetAttributes(@"C:\ReadMe.txt");

         // Falls wir die Attribute bekommen haben, war die Methode erfolgreich.
         success = true;
      }
      catch (SecurityException)
      {
         // Methode ist aufgrund ungengender Sicherheitsberechtigungen
         // im Ausfhrungskontext des Threads fehlgeschlagen.
      }

      // Anzeigen, ob der Zugriffsversuch erfolgreich war oder fehlgeschlagen ist.
      Console.WriteLine("{0}: {1}", test, success);
   }
}
