/*=====================================================================
  File:      Thread.cs

  Summary:   Demonstrates how to create and manipulate threads.

---------------------------------------------------------------------
  This file is part of the Microsoft COM+ 2.0 SDK Code Samples.

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/


// Look up thread sync'd Console WriteLine's
/*
Simple.cool
StopJoin.cool
SyncOne: Shows two threadds sharing 1 object using CriticalSections
*/
/*
Show how to spawn a thread using ThreadStart
Show how to get the thread's state
Show how to query the thread's ID (hash code)
Show how to: Join, Sleep, Start, Suspend, Interrupt, Resume, Stop (how does this differ from stop other than raising a different exception class)

Show these properties:
    CurrentThread  (Static)
    Name  - for debugging
    ThreadState

    IsAlive  
    IsBackground - If false, app won't shutdown until thread dies. If true, app can shutdown forcibly killing the background threads
    Priority  
    ApartmentState????
*/
/*
SYNCHRONIZATION SAMPLE
Object and the SyncBlock
Each object has an associated SyncBlock index. If an object has not been used in a synchronization operation then the associated SyncBlock index 
is empty. When the object is used in a synchronization operation then a SyncBlock is retrieved from the SyncBlock cache. The SyncBlock is 
associated with the object by updating the objects SyncBlock index.

A SyncBlock is a sparsely allocated data structure.  It currently serves several purposes: 
It contains the lock used to implement synchronized methods and synchronized blocks of code.

It contains the list of waiting threads used to implement Object.Wait(), Object.Notify() and Object.NotifyAll(). 
Generally most objects will not have SyncBlocks, unless a synchronization operation has performed against the object. 
Furthermore, the SyncBlock is accessed using a SyncBlockIndex which is allocated at a negative offset on the managed object. 
All objects have a SyncBlockIndex, but most of these indices have a value of 0 - meaning they share a single dummy SyncBlock. Logically, 
such objects dont have a SyncBlock. However, theres a set of objects that have non-zero SyncBlockIndexes, but they dont actually 
have SyncBlocks. These are objects that dont otherwise need a SyncBlock, but which have had their Object.GetHashCode() method called. 
If a class uses the default (runtime provided) implementation of GetHashCode(), this method will allocate a SyncBlockIndex but will not 
allocate the SyncBlock (to save memory).  The result is a relatively unique and stable hash code.
It is only relatively unique because SyncBlockIndexes will be recycled when objects are collected by GC, and because other classes 
which override GetHashCode() can certainly return the same numbers. Also, its relatively stable because it does not persist with the object. 
It is only valid so long as the object is activated within a particular EE instance.
Show Object .Notify, .NotifyAll, .Wait

Show how to have multiple threads accessing a single object
Show Interlocked class
Show ManualResetEvent/AutoResetEvent
Show ReadWriterLock class
Sync the object with CriticalSection
Sync the object with "Synchronized on the method". Show additional Cool means?
Show ThreadPools, Timers, and WaitHandles?


Show separate sample on TLS? number slot, name slot, secure slot?
    Static methods: AllocateDataSlot/FreeDataSlot, AllocateNamedDataSlot/FreeNamedDataSlot, AllocateSecureDataSlot/FreeSecureDataSlot
    Static Set/Get data: SetData/GetData, SetData/GetData, SetSecureData/GetSecureData

Understand these better: 
    GetCurrentContext Gets the current context from the slot. 
    SetCurrentContext Sets the Context to a specified value and returns the old Context. 

    GetDomain Returns the current domain in which the current thread is running.    
    GetDomainFromDomainID To be supplied. 
    GetDomainID To be supplied.  

    PopDomain Removes the domain the thread is currently part of. 
    PushDomain Associates this thread with a specified domain. 

    GetThreadContext Returns the context ID within which the current thread is executing. 
*/


using System;
using System.Threading;
using System.Text;


class BackgroundTask {
    Thread ParentThread;
    public Boolean Stop = false;

    public BackgroundTask(Thread parentThread) { 
        ParentThread = parentThread;
    }
}


class Application {
    static public Boolean Stop = false;
    static public void ThreadFunc() {
      try {
         try {
            Thread.Sleep(1000);
            while (!Stop) {
               Console.WriteLine("Second thread running");
            }
         }
         finally {
            Console.WriteLine("Reset abort");
            Thread.ResetAbort();
         }
      }
      catch (ThreadAbortException e) {
         //Thread.ResetAbort();
         //Console.WriteLine("Exception State: " + e.ExceptionState.GetType());
         Console.WriteLine(e);
      }
      Console.WriteLine("Second thread terminating normally");
      return;

      #if NO
        Monitor.Enter(ParentThread);
        Thread SecondThread = Thread.CurrentThread;
        Console.WriteLine("Second thread ID: "   + SecondThread.GetHashCode());
        Console.WriteLine("Second thread Name: " + SecondThread.Name);
        Monitor.Exit(ParentThread);
        while (!Stop) {
            Console.WriteLine("Second thread running");
        }

        Console.WriteLine("Second thread starting to sleep");
        try {
            Thread.Sleep(2000);
        }
        catch (ThreadInterruptedException) {
            Console.WriteLine("   Second thread was interrupted. Ignoring.");
        }
        Console.WriteLine("Second thread done sleeping");
        try {
            while (true) {
                Console.WriteLine("Second thread running");
            }
        }
        catch (ThreadStopException) {
            Console.WriteLine("   Second thread was stopped. Ignoring.");
        }
        Console.WriteLine("Second thread still running");
      #endif
    }
public static void f() {

      lock(typeof(Application)) {}
      object o = typeof(Application);
      Monitor.Enter(o);
      try {
      }
      finally {
         Monitor.Exit(o);
      }
}

    public static void Main(String[] args) {
      Object o = new Object();
      Thread[] t = new Thread[5];
      for (int n = 0; n < 5; n++) {
         Count c = new Count();
         c.n = n;
         c.o = o;
         t[n] = new Thread(new ThreadStart(c.CountPart));
         t[n].Start();
      }
      WaitHandle[] h = new WaitHandle[t.Length];
      for (int n = 0; n < t.Length; n++) {
         t[n].Join();//h[n] = t.
      }
   }


   public static void CountPart(Object o, int n) {

      Console.WriteLine(o.GetHashCode());
      lock (o) {
         Console.WriteLine(o.GetHashCode());
      }
//    return;


      
      Thread t = new Thread(new ThreadStart(ThreadFunc));
        Console.WriteLine("2nd thread state: " + t.ThreadState.Format());
      t.Start();
      t.Start();
      Thread.Sleep(500);
      t.Abort("Jeff");
         Console.WriteLine("2nd thread state: " + t.ThreadState.Format());
      //t.Interrupt();
      for (int x = 0; x < 100; x++) {
         Console.WriteLine("2nd thread state: " + t.ThreadState.Format());
      }
      Console.WriteLine("Stopping 2nd thread" );
      //Stop = true;
      //t.Interrupt();
      for (int x = 0; x < 100; x++) {
         Console.WriteLine("2nd thread state: " + t.ThreadState.Format());
      }
      t.Join();
      t.Interrupt();


#if NO
        Thread MainThread = Thread.CurrentThread;
        MainThread.Name = "MainThread";
        Console.WriteLine("Main thread ID: " + MainThread.GetHashCode());
        Console.WriteLine("Main thread Name: " + MainThread.Name);
        Console.WriteLine("Main thread state: " + MainThread.ThreadState.Format());
        Console.WriteLine("Main thread apartment state: " + MainThread.ApartmentState.Format());
        Console.WriteLine("Main thread CurrentCulture: " + MainThread.CurrentCulture);
        Console.WriteLine("Main thread CurrentUICulture: " + MainThread.CurrentUICulture);
        Console.WriteLine("Main thread IsAlive: " + MainThread.IsAlive);
        Console.WriteLine("Main thread IsBackground: " + MainThread.IsBackground);
        Console.WriteLine("Main thread Priority: " + MainThread.Priority);


      // Static properties
      Console.WriteLine("Main thread CurrentContext: " + Thread.CurrentContext);
        Console.WriteLine("Main thread CurrentPrincipal: " + Thread.CurrentPrincipal);
        Console.WriteLine("Main thread CurrentThread: " + Thread.CurrentThread);


        BackgroundTask bt = new BackgroundTask(MainThread);
        Thread SecondThread = new Thread(new ThreadStart(bt.ThreadFunc));
        Console.WriteLine("Main thread apartment state: " + SecondThread.ApartmentState.Format());
        SecondThread.ApartmentState = ApartmentState.MTA;
        Console.WriteLine("Main thread apartment state: " + SecondThread.ApartmentState.Format());
        SecondThread.ApartmentState = ApartmentState.STA;
        Console.WriteLine("Main thread apartment state: " + SecondThread.ApartmentState.Format());
      SecondThread.Start();
      SecondThread.Join();
        Console.WriteLine("After Join");
        SecondThread.ApartmentState = ApartmentState.STA;
        Console.WriteLine("Second thread state: " + SecondThread.ThreadState.Format());

      return;
#endif
#if NO
        Console.WriteLine("Main thread state: " + ThreadStateToString(MainThread.ThreadState) + "\n");
        BackgroundTask bt = new BackgroundTask(MainThread);
        Thread SecondThread = new Thread(new ThreadStart(bt.ThreadFunc));
        SecondThread.Name = "SecondThread";
        Console.WriteLine("Second thread ID: "   + SecondThread.GetHashCode());
        Console.WriteLine("Second thread Name: " + SecondThread.Name);
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));

        Monitor.Enter(MainThread);
        SecondThread.Start();
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
        while ((SecondThread.ThreadState & ThreadState.Unstarted) != 0) {  // Jeff: This is more accurate than IsAlive
            Console.WriteLine("Second thread not started yet");
            Thread.Sleep(1);
        }
        Thread.Sleep(500);
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
        Console.ReadLine();
        Monitor.Exit(MainThread);

        // NOTE: Suspend is NOT like Win32's SuspendThread. Runtime threads are only suspended at specific locations
        SecondThread.Suspend();
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
        while ((SecondThread.ThreadState & ThreadState.Suspended) == 0) {  // Wait till Second thread is suspended (JEFF: How do they know?)
            Console.WriteLine("Second thread not suspended yet");
            Thread.Sleep(1);
        }
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
        bt.Stop = true;

        SecondThread.Interrupt();  // Attempt to kill the second thread
        SecondThread.Resume();  // Resume it again
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
        Thread.Sleep(1000);
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
        SecondThread.Stop();
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
        SecondThread.Join();
        Console.WriteLine("Join returned");
        Console.WriteLine("Second thread state: "   + ThreadStateToString(SecondThread.ThreadState));
/*
        Console.WriteLine("Current context = " + t.GetCurrentContext());
        Console.WriteLine("Thread context = " + Thread.GetThreadContext());
        Console.WriteLine("AppDomain = " + Thread.GetDomain());
        Console.WriteLine("AppDomain ID = " + Thread.GetDomainID());
        Console.WriteLine("AppDomain from ID = " + Thread.GetDomainFromDomainID(Thread.GetDomainID()));
        Console.WriteLine("GetPriority= " + t.Priority);
*/
#endif
    } 
}


///////////////////////////////////////////////////////////////////////////////


class Count {
   static private int num = 0;
   public int n;
   public Object o;

   public void CountPart() {
      lock(o) {
         for (int x = 0; x < 10; x++) {
             if (num == n) {
               Console.Write(((n == 0) ? "\n" : "") + n + " ");
               num = ++num % 5;
               Monitor.Pulse(o);
            }
            Monitor.Wait(o);
         }
      }
   }     
}


///////////////////////////////////////////////////////////////////////////////


class Handshake {
   public class HandshakeData { }
   Object objRequestReady  = new Object();
   Object objResponseReady = new Object();
   HandshakeData Request  = null;
   HandshakeData Response = null;

   public HandshakeData MakeRequest(HandshakeData request) {
      HandshakeData Response;
      lock(this) {   // Only 1 request at a time
         lock(objRequestReady) { 
            // The calling client can make a request
            this.Request = request;
            this.Response = null;   // Get ready

            // Wake a server (that is hopefully listening)
            Monitor.Pulse(objRequestReady);
         }

         lock(objResponseReady) {
            if (this.Response == null) {
               // No response yet, wait for it
               Monitor.Wait(objResponseReady);
            }
            // Save Response in local for return
            Response = this.Response;
                this.Request = null;
         }  // lock Response
      }     // lock this
      return(Response);
   }


   public void ProcessRequest() {
      lock(objRequestReady) {
         while (this.Request == null) {
            // No request pending, wait for it
            Monitor.Wait(objRequestReady);
         }
         // Process the request
         // ...

         // Prepare the response
         lock(objResponseReady) {
            this.Response = new HandshakeData();
            Monitor.Pulse(objResponseReady); // In case client is waiting
         }
      }
   }
}


///////////////////////////////////////////////////////////////////////////////


class Queue {
   private int numItems = 0;
   private int numWaiting = 0;
   void EnqueueItem(QueueElement qe) {
      lock(this) {
         // Add the QueueElement (qe) to the queue...
         Monitor.Pulse(this); // Wakeup a waiter if one is waiting
         numItems++;
      }
   }

   QueueElement DequeueItem() {
      QueueElement qe = null;
      lock(this) {
         if (numItems == 0 || numWaiting > 0) {
            numWaiting++;
            // No items, wait for an item
            Monitor.Wait(this);
            numWaiting--;
         }
         // Remove a QueueElement (qe) from the queue...
         numItems--;
      }
      return(qe);
   }
}


///////////////////////////////////////////////////////////////////////////////


class Queue {
   // Allows mutual-exclusive access to the Q's data structures
   Mutex QAccess = CreateMutex();   

   // Indicates the number of items in the queue (initialized to 0)
   Semaphore QNotEmpty = CreateSemaphore(0, Int32.MaxValue);

   void EnqueueItem(QueueElement qe) {
      QAccess.WaitOne();            // Obtain exclusive access to the queue
      // Add the QueueElement (qe) to the queue...
      QNotEmpty.ReleaseSemaphore(); // Increment number of items in the queue
      QAccess.ReleaseMutex();       // Release exclusive access to the queue
   }

   QueueElement DequeueItem() {
      QueueElement qe = null;

      // Wait until there is at least 1 item in the queue and until
      // we can get exclusive access to the queue
      WaitHandle.WaitAll(new WaitHandle[] { QAccess, QNotEmpty });

      // Remove a QueueElement (qe) from the queue...

      QAccess.ReleaseMutex();       // Release exclusive access to the queue
      return(qe);
   }
}


///////////////////////////////////////////////////////////////////////////////
