using System;
using System.Threading;

public sealed class Program {
   public static void Main() {
   }

   private void SomeMethodWithLock() {
      lock (this) {
         // Hier kann threadsicherer Code ausgefhrt werden...
      }  
   }

   private void SomeMethodAfterLockTranslation() {
      Object temp = this;
      Monitor.Enter(temp);
      try {
         // Hier kann threadsicherer Code ausgefhrt werden...
      }
      finally {
         Monitor.Exit(temp);
      }
   }
}

internal sealed class Transaction
{

   // Dieses Feld gibt an, wann die letzte Transaktion ausgefhrt wurde.
   private DateTime timeOfLastTransaction;

   public void PerformTransaction()
   {
      Monitor.Enter(this); // In die Sperre dieses Objekts eintreten.
      // Die Transaktion ausfhren...

      // Zeit der letzten Transaktion vermerken.
      timeOfLastTransaction = DateTime.Now;

      Monitor.Exit(this);  // Die Sperre dieses Objekts verlassen.
   }

   // Eine schreibgeschtzte Eigenschaft, die die Zeit der letzten Transaktion zurckgibt.
   public DateTime LastTransaction
   {
      get
      {
         Monitor.Enter(this); // In die Sperre dieses Objekts eintreten.

         // Zeit der letzten Transaktion
         // in einer temporren Variablen speichern.
         DateTime dt = timeOfLastTransaction;

         Monitor.Exit(this);  // Die Sperre dieses Objekts verlassen.
         return dt; // Gespeicherte Zeit zurckgeben.
      }
   }
}


internal sealed class TransactionWithLock {

   // Dieses Feld gibt an, wann die letzte Transaktion ausgefhrt wurde.
   private DateTime timeOfLastTransaction;

   public void PerformTransaction()
   {
      lock (this)
      {  // In die Sperre dieses Objekts eintreten.
         // Die Transaktion ausfhren...

         // Zeit der letzten Transaktion vermerken.
         timeOfLastTransaction = DateTime.Now;
      }  // Die Sperre dieses Objekts verlassen.
   }

   // Eine schreibgeschtzte Eigenschaft, die die Zeit der letzten Transaktion zurckgibt.
   public DateTime LastTransaction
   {
      get
      {
         lock (this)
         {  // In die Sperre dieses Objekts eintreten.
            return timeOfLastTransaction; // Gespeicherte Zeit zurckgeben.
         }  // Die Sperre dieses Objekts verlassen.
      }
   }
}

internal static class StaticTransactionWithLock {

   // Dieses Feld gibt an, wann die letzte Transaktion ausgefhrt wurde.
   private static DateTime timeOfLastTransaction;

   public static void PerformTransaction()
   {
      lock (typeof(Transaction))
      {  // In die Sperre des Typobjekts eintreten.
         // Die Transaktion ausfhren...

         // Zeit der letzten Transaktion vermerken.
         timeOfLastTransaction = DateTime.Now;
      }  // Die Sperre des Typobjekts verlassen.
   }

   // Eine schreibgeschtzte Eigenschaft, die die Zeit der letzten Transaktion zurckgibt.
   public static DateTime LastTransaction
   {
      get
      {
         lock (typeof(Transaction))
         {  // In die Sperre des Typobjekts eintreten.
            return timeOfLastTransaction; // Gespeicherte Zeit zurckgeben.
         }  // Die Sperre des Typobjekts verlassen.
      }
   }
}

internal sealed class TransactionWithLockObject {
   // Ein privates Objekt als Sperrobjekt anlegen.
   private Object m_lock = new Object();

   // Dieses Feld gibt an, wann die letzte Transaktion ausgefhrt wurde.
   private DateTime timeOfLastTransaction;

   public void PerformTransaction()
   {
      lock (m_lock)
      {  // In die Sperre des Objekts im privaten Feld eintreten.
         // Die Transaktion ausfhren...

         // Zeit der letzten Transaktion vermerken.
         timeOfLastTransaction = DateTime.Now;
      }  // Die Sperre des Objekts im privaten Feld verlassen.
   }

   // Eine schreibgeschtzte Eigenschaft, die die Zeit der letzten Transaktion zurckgibt.
   public DateTime LastTransaction
   {
      get
      {
         lock (m_lock)
         {  // In die Sperre des Objekts im privaten Feld eintreten.
            return timeOfLastTransaction; // Gespeicherte Zeit zurckgeben.
         }  // Die Sperre des Objekts im privaten Feld verlassen.
      }
   }
}
