using System;
using System.Threading;

public static class Program {
   public static void Main() {
      Singleton s = Singleton.Value;
   }
}

public sealed class Singleton
{
   // s_lock wird fr die Threadsicherheit bentigt. Da wir dieses Objekt
   // haben, wird angenommen, dass es aufwendiger ist, das Singleton-Objekt
   // anzulegen als ein System.Object-Objekt, und dass es mglicherweise
   // berhaupt nicht ntig ist, das Singleton-Objekt anzulegen. Andernfalls
   // wre es effizienter und einfacher, einfach das Singleton-Objekt
   // in einem Klassenkonstruktor anzulegen
   private static Object s_lock = new Object();

   // Das Schlsselwort volatile wird beim Speichermodell der CLR nicht bentigt, aber
   // fr das ECMA-Speichermodell. Sicher ist sicher, deshalb habe ich es eingefgt.
   private static volatile Singleton s_value;

   // Der private Konstruktor verhindert, dass beliebiger Code auerhalb
   // dieser Klasse eine Instanz anlegen kann.
   private Singleton() { }

   // Eine ffentliche, statische Eigenschaft, die das Singleton-Objekt
   // zurckgibt (und es bei Bedarf anlegt).
   public static Singleton Value
   {
      get
      {
         // Wurde das Singleton-Objekt bereits angelegt?
         if (s_value == null)
         {
            // Nein, nur ein Thread sollte es anlegen.
            lock (s_lock)
            {
               // Hat ein anderer Thread es angelegt?
               if (s_value == null)
               {
                  // Nein. Gut, dann legt dieser Thread es an.

                  // Das Schlsselwort volatile stellt sicher, dass alle Felder des
                  // Singleton-Objekts (die im Konstruktor initialisiert wurden) in
                  // den Arbeitsspeicher geschrieben werden, bevor andere Threads
                  // den Verweis auf das Singleton-Objekt zu sehen bekommen.
                  s_value = new Singleton();
               }
            }
         }

         // Einen Verweis auf das Singleton-Objekt zurckgeben.
         return s_value;
      }
   }
}


namespace SimplifiedSingleton {
   public sealed class Singleton {
      private static Singleton s_value = new Singleton();

      // Der private Konstruktor verhindert, dass beliebiger Code auerhalb
      // dieser Klasse eine Instanz anlegen kann.
      private Singleton() { }

      // Eine ffentliche, statische Eigenschaft, die das Singleton-Objekt zurckgibt.
      public static Singleton Value
      {
         get
         {
            return s_value;  // Einen Verweis auf das Singleton-Objekt zurckgeben.
         }
      }
   }
}