using System;
using System.Threading;

public static class Program {
   public static void Main(string[] args) {

   }
}

internal sealed class QueueItem { }

internal sealed class Queue {
   private Int32  m_numItems = 0;   // Anzahl der QueueItem-Objekte in der Warteschlange
   private Int32  m_numWaiting = 0; // Anzahl der Threads, die DequeueItem aufrufen
   private Object m_lock = new Object();

   // Methode zum Hinzufgen eines QueueItem zur Warteschlange
   public void EnqueueItem(QueueItem qi) {
      lock (m_lock) {
         // (Hier einfgen: Code zum Hinzufgen von qi zur Warteschlange...)
         m_numItems++;
         Monitor.Pulse(m_lock); // Wartendes Objekt aufwecken, sofern vorhanden
      }
   }

   // Methode zum Entfernen eines QueueItem aus der Warteschlange
   public QueueItem DequeueItem() {
      QueueItem qi = null;
      lock (m_lock) {
         if (m_numItems == 0 || m_numWaiting > 0) {
            // Keine Elemente, auf eines warten
            m_numWaiting++;
            Monitor.Wait(m_lock);
            m_numWaiting--;
         }
         // (Hier einfgen: Code zum Entfernen eines QueueItem ...)
         m_numItems--;
      }
      return qi;
   }
}

internal sealed class QueueWithKernelObjects {
   // Zugriff mit gegenseitigem Ausschluss auf die Datenstrukturen von Q
   Mutex QAccess = new Mutex();

   // Gibt die Anzahl der Elemente in der Warteschlange an (Anfangswert 0)
   Semaphore QNotEmpty = new Semaphore(0, Int32.MaxValue);

   public void EnqueueItem(QueueItem qi) {
      QAccess.WaitOne();            // Exklusiven Zugriff anfordern
      // Das QueueItem (qi) zur Warteschlange hinzufgen...
      QNotEmpty.Release();          // Anzahl der Elemente erhhen
      QAccess.ReleaseMutex();       // Exklusiven Zugriff abgeben
   }

   public QueueItem DequeueItem() {
      QueueItem qi = null;

      // Warten, bis zumindest ein Element in der Warteschlange steht und
      // wir exklusiven Zugriff auf die Warteschlange erhalten knnen.
      WaitHandle.WaitAll(new WaitHandle[] { QAccess, QNotEmpty });

      // Ein QueueItem (qi) aus der Warteschlange entfernen...

      QAccess.ReleaseMutex();       // Exklusiven Zugriff abgeben
      return qi;
   }
}
