using System;
using System.Threading;

public static class Program {
   public static void Main() {
      // Ein AutoResetEvent anlegen (anfangs nicht signalisiert)
      AutoResetEvent are = new AutoResetEvent(false);

      // Den Threadpool anweisen, dass er auf das AutoResetEvent warten soll.
      RegisteredWaitHandle rwh = ThreadPool.RegisterWaitForSingleObject(
         are,             // Warten auf dieses AutoResetEvent.
         EventOperation,  // Diese Methode zurckrufen.
         null,            // null fr EventOperation bergeben.
         5000,            // 5 Sekunden darauf warten, dass das Ereignis signalisiert wird.
         false);          // EventOperation jedes Mal aufrufen, wenn das Ereignis signalisiert wird.

      // Unsere Schleife starten.
      Char operation;
      do
      {
         Console.WriteLine("S=Signal, Q=Quit?");
         operation = Char.ToUpper(Console.ReadKey(true).KeyChar);
         if (operation == 'S')
         {
            // Benutzer will das Ereignis signalisieren. Wir setzen es.
            are.Set();
         }
      } while (operation != 'Q');

      // Dem Threadpool mitteilen, dass er nicht mehr auf das Ereignis warten soll.
      rwh.Unregister(null);
   }

   // Diese Methode wird immer aufgerufen, wenn das Ereignis signalisiert wird oder
   // wenn 5 Sekunden seit dem letzten Signal/Timeout vergangen sind.
   private static void EventOperation(Object state, Boolean timedOut)
   {
      if (timedOut)
      {
         Console.WriteLine("Timeout beim Warten auf das AutoResetEvent.");
      }
      else
      {
         Console.WriteLine("Das AutoResetEvent wurde signalisiert.");
      }
   }
}
